/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.cms.test;

import com.digt.trusted.asn1.ASN1InputStream;
import com.digt.trusted.asn1.cms.ContentInfo;
import com.digt.trusted.asn1.cms.EnvelopedData;
import com.digt.trusted.cms.*;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.security.KeyPair;
import java.security.cert.CertStore;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.X509Certificate;
import java.util.*;

/**
 * User: andy
 * Date: 06.06.2005
 * Time: 11:14:33
 */
public class PerformanceTest
        extends TestCase {

    /*
    *
    *  VARIABLES
    *
    */
    boolean DEBUG = true;
    static final int testsCount = 5;
    static final int threadCount = 50;
    static final int signLen = 64;
    static final int GOST3410 = 1;
    static final int GOST3410EL = 2;
    static final int GOSTDH = 3;
    static final int GOSTECDH = 4;

    String data;
    String subDN;
    String issDN;
    KeyPair issKP;
    KeyPair subKP;
    X509Certificate devCert;

    private static int threads = 0;

    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public PerformanceTest(String name) {
        super(name);
    }

    public static void main(String args[]) {

        junit.textui.TestRunner.run(PerformanceTest.class);
    }

    public static junit.framework.Test suite() {
        return new TestSuite(PerformanceTest.class);
    }

    public void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        try {
            data = "Test data";
            subDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            issDN = "CN=CA, E=info@digt.ru, O=DIGT, C=RU";
            issKP = makeKeyPair(GOST3410EL);
            subKP = makeKeyPair(GOSTECDH);
            devCert = KeyPairAndCertificateUtil.makeCertificate(subKP, subDN, issKP, issDN, "GOST3411WithGOST3410");
        } catch (Exception ex) {
            log(ex);
            fail();
        }

    }

    public void tearDown() {
    }

    /*
    *
    *  TESTS
    *
    */


/*
    public void testSignedData() {
        try {
            String _data = "Test data";
            String _subDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            String _issDN = "CN=CA, E=info@digt.ru, O=DIGT, C=RU";
            KeyPair _issKP = makeKeyPair(GOST3410);
            KeyPair _subKP = makeKeyPair(GOST3410);
            X509Certificate _devCert = KeyPairAndCertificateUtil.makeCertificate(_subKP, _subDN, _issKP, _issDN, "GOST3411WithGOST3410", false);
            long _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                SignedData(_subKP, _devCert, _data);
            }
            log(runTime("Stress Test for SignedData ", _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testEnvelopedData() {
        try {
            String _data = "Test data";
            String _subDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            String _issDN = "CN=CA, E=info@digt.ru, O=DIGT, C=RU";
            KeyPair _issKP = makeKeyPair(GOST3410);
            KeyPair _subKP = makeKeyPair(GOSTDH);
            X509Certificate _devCert = KeyPairAndCertificateUtil.makeCertificate(_subKP, _subDN, _issKP, _issDN, "GOST3411WithGOST3410", false);
            long _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                EnvelopedData(_subKP, _devCert, _data);
            }
            log(runTime("Stress Test for EnvelopedData ", _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }

    }

*/
    public void testMultiThread() {
        new TestThread().runTest();
    }

    /*
    *
    *  INTERNAL METHODS
    *
    */

    public void SignedData(KeyPair _signKP, X509Certificate _signCert, String _signData) {
        try {
            //  
            // 
            CMSProcessable msg = new CMSProcessableByteArray(_signData.getBytes());
            // CMS 
            CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
            // 
            gen.addSigner(_signKP.getPrivate(), _signCert, CMSSignedDataGenerator.DIGEST_GOST3411);

            //    CRL    ,        CRL ,       ,    .
            ArrayList certList = new ArrayList();
            certList.add(_signCert);
            CertStore certs = CertStore.getInstance("Collection",
                    new CollectionCertStoreParameters(certList), "DIGT");
            gen.addCertificatesAndCRLs(certs);

            //
            CMSSignedData s = gen.generate(msg, true, "DIGT");
            //  
            ByteArrayInputStream bIn = new ByteArrayInputStream(s.getEncoded());
            //    .
            /*DataOutputStream out_msg = new DataOutputStream( new BufferedOutputStream(new FileOutputStream("c:\\signed_gost.p7s.")));
            for (int i = 0; i != bIn.length; i++) out_msg.writeByte(bIn[i]);
            out_msg.close();*/

            //  
            // ASN   
            ASN1InputStream aIn = new ASN1InputStream(bIn);

            // CMS   ASN 
            s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));

            //     ,           
            certs = s.getCertificatesAndCRLs("Collection", "DIGT");

            //    
            SignerInformationStore signers = s.getSignerInfos();
            Collection c = signers.getSigners();
            Iterator it = c.iterator();
            //   
            while (it.hasNext()) {
                //    
                SignerInformation signer = (SignerInformation) it.next();
                //     
                Collection certCollection = certs.getCertificates(signer.getSID());
                Iterator certIt = certCollection.iterator();
                X509Certificate cert = (X509Certificate) certIt.next();
                // .
                log("Signature verification status = " + signer.verify(cert, "DIGT"));
                assertEquals(true, signer.verify(cert, "DIGT"));
            }
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    private void EnvelopedData(KeyPair _recipientKP, X509Certificate _recipientCert, String _data) {

        try {

            CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
            edGen.addKeyTransRecipient(_recipientCert);

            CMSEnvelopedData ed = edGen.generate(new CMSProcessableByteArray(_data.getBytes()),
                    CMSEnvelopedDataGenerator.GOST28147_CFB, "DIGT");

            //encoding
            byte[] ec = ((EnvelopedData) ed.getcontentInfo().getContent()).
                    getEncryptedContentInfo().getGOSTEncryptedContent();
            //output to file p7m.
            /*DataOutputStream out = new DataOutputStream( new BufferedOutputStream(new FileOutputStream("c:\\gost_enveloped.p7m")));
            for (int i = 0; i != ec.length; i++)  out.writeByte(ec[i]);
            out.close();*/
            DataInputStream in = new DataInputStream(new ByteArrayInputStream(ec));

            RecipientInformationStore recipients = ed.getRecipientInfos();

            Collection c = recipients.getRecipients();
            Iterator it = c.iterator();

            while (it.hasNext()) {
                RecipientInformation recipient = (RecipientInformation) it.next();
                byte[] recData = recipient.getGOSTContent(_recipientKP.getPrivate(), "DIGT", in);
                log("before encrypting = " + _data + " after decrypting = " + new String(recData));
                assertEquals(true, Arrays.equals(_data.getBytes(), recData));
            }
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    private KeyPair makeKeyPair(int _algo) {
        switch (_algo) {
            case GOST3410:
                return KeyPairAndCertificateUtil.makeGOST3410KeyPair();
            case GOST3410EL:
                return KeyPairAndCertificateUtil.makeGOST3410ELKeyPair();
            case GOSTDH:
                return KeyPairAndCertificateUtil.makeGOSTDHKeyPair();
            case GOSTECDH:
                return KeyPairAndCertificateUtil.makeGOSTELDHKeyPair();
            default:
                return null;
        }
    }

    private String getAlgo(int _algo) {
        switch (_algo) {
            case GOST3410:
                return "GOST3410";
            case GOST3410EL:
                return "GOST3410EL";
            case GOSTDH:
                return "GOSTDH";
            case GOSTECDH:
                return "GOSTECDH";
            default:
                return "";
        }
    }


    private String runTime(String _testName, long _start) {
        long _runTime = getTime() - _start;
        return _testName + " was run " + testsCount + " times  in " + _runTime / 1000 + " second.";
    }

    private long getTime() {
        Calendar cal = Calendar.getInstance();
        return cal.getTimeInMillis();
    }

    private void Signed(int _thread) {
        try {
            long _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                SignedData(subKP, devCert, data);
            }
            log(runTime("SignedData Stress Test for Thread Number " + _thread , _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }

    }

    private void Enveloped(int _thread) {
        try {
            long _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                EnvelopedData(subKP, devCert, data);
            }
            log(runTime("EnvelopedData Stress Test for Thread Number " + _thread , _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }

    }

    private class TestThread extends Thread {

        public void run() {
            int _thread = ++threads;
            log("Thread Number " + _thread + " started");
            Signed(_thread);
            Enveloped(_thread);
            log("Thread Number " + _thread + " stopped");
            --threads;
        }

        public void runTest() {
            try {
                for (int i = 0; i < threadCount; i++) {
                    new TestThread().start();
                }
                sleep(1000);
                while(threads>0) {
                    sleep(100);
                }
                log("Test finished");
            } catch (Exception ex) {
                log(ex);
                fail();
            }
        }

    }

}
