/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.cms.test;

import java.io.ByteArrayInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.security.KeyPair;
import java.security.Security;
import java.security.cert.CertStore;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.digt.trusted.asn1.ASN1InputStream;
import com.digt.trusted.asn1.DERObjectIdentifier;
import com.digt.trusted.asn1.cms.ContentInfo;
import com.digt.trusted.asn1.x509.X509ObjectIdentifiers;
import com.digt.trusted.cms.CMSProcessable;
import com.digt.trusted.cms.CMSProcessableByteArray;
import com.digt.trusted.cms.CMSSignedData;
import com.digt.trusted.cms.CMSSignedDataGenerator;
import com.digt.trusted.cms.SignerInformation;
import com.digt.trusted.cms.SignerInformationStore;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.util.encoders.Base64;

public class SignedDataTest
        extends TestCase
{

     /*
    *
    *  VARIABLES
    *
    */

    boolean DEBUG = true;
    DERObjectIdentifier hashAlg;
    String          sigAlg;
    String          signDN;
    KeyPair         signKP;
    X509Certificate signCert;


    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public SignedDataTest(String name) {
        super(name);
    }

    public static void main(String args[]) {

        junit.textui.TestRunner.run(SignedDataTest.class);
    }

    public static Test suite() {
        return new TestSuite(SignedDataTest.class);
    }

    public void log(Exception _ex) {
        if(DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if(DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp()
    {
        try
        {
            signDN   = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
        }
        catch(Exception ex)
        {
            log(ex);
            fail();
        }
    }

    public void tearDown() {

    }

    /*
    *
    *  TESTS
    *
    */

    
    //      
    protected void createSignAndVerifyIt()
    {
        try
        {
            //  
            // 
            CMSProcessable      msg = new CMSProcessableByteArray("Test Data".getBytes());

            //    CRL    ,        CRL ,       ,    .
            ArrayList           certList = new ArrayList();
            certList.add(signCert);

            CertStore           certs = CertStore.getInstance("Collection",
                    new CollectionCertStoreParameters(certList), "DIGT");

            // CMS 
            CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
            // 
            gen.addSigner(signKP.getPrivate(), signCert, hashAlg.getId());

            gen.addCertificatesAndCRLs(certs);

            //
            CMSSignedData s = gen.generate(msg, true, "DIGT");
            //  
            ByteArrayInputStream bIn = new ByteArrayInputStream(s.getEncoded());

            //    .
            /*
            FileOutputStream out_msg = new FileOutputStream("c:\\signed_gost.sig");
            out_msg.write(s.getEncoded());
            out_msg.close();
            */

            //  
            // ASN   
            ASN1InputStream      aIn = new ASN1InputStream(bIn);

            // CMS   ASN 
            s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));

            //     ,           
            certs = s.getCertificatesAndCRLs("Collection", "DIGT");

            //    
            SignerInformationStore  signers = s.getSignerInfos();
            Collection              c = signers.getSigners();
            Iterator                it = c.iterator();
            //   
            while (it.hasNext())
            {
                //    
                SignerInformation   signer = (SignerInformation)it.next();
                //     
                Collection          certCollection = certs.getCertificates(signer.getSID());

                Iterator        certIt = certCollection.iterator();
                X509Certificate cert = (X509Certificate)certIt.next();
                // .
                log("Signature verification status = " + signer.verify(cert, "DIGT"));
                assertEquals(true, signer.verify(cert, "DIGT"));
            }
        }
        catch(Exception ex)
        {
            log(ex);
            fail();
        }
    }

    
//    //      "GOST3411WithGOST3410"
//    public void testGOST3411WithGOST3410()
//    {
//        try
//        {
//            //      
//            hashAlg = X509ObjectIdentifiers.gostR3411;
//            sigAlg   = "GOST3411WithGOST3410";
//            signKP   = KeyPairAndCertificateUtil.makeGOST3410KeyPair();
//            signCert = KeyPairAndCertificateUtil.makeCertificate(signKP, signDN, signKP, signDN, sigAlg);
//            
//            createSignAndVerifyIt();
//        }
//        catch(Exception ex)
//        {
//            log(ex);
//            fail();
//        }
//    }

    //      "GOST3411WithGOST3410EL"
    public void testGOST3411WithGOST3410EL()
    {
        try
        {
            //      
            hashAlg = X509ObjectIdentifiers.gostR3411;
            String   sigAlg   = "GOST3411WithGOST3410EL";
            signKP   = KeyPairAndCertificateUtil.makeGOST3410ELKeyPair();
            signCert = KeyPairAndCertificateUtil.makeCertificate(signKP, signDN, signKP, signDN, sigAlg);

            createSignAndVerifyIt();
        }
        catch(Exception ex)
        {
            log(ex);
            fail();
        }
    }

    //      "GOST3411v12256WithGOST3410v12256"
    public void testGOST3411v12256WithGOST3410v12256()
    {
        try
        {
            //      
            hashAlg  = X509ObjectIdentifiers.id_tc26_gost3411_12_256;
            sigAlg   = "GOST3411v12256WithGOST3410v12256";
            signKP   = KeyPairAndCertificateUtil.makeGOST3410v12256KeyPair();
            signCert = KeyPairAndCertificateUtil.makeCertificate(signKP, signDN, signKP, signDN, sigAlg);

            createSignAndVerifyIt();
        }
        catch(Exception ex)
        {
            log(ex);
            fail();
        }
    }

    //      "GOST3411v12512WithGOST3410v12512"
    public void testGOST3411v12512WithGOST3410v12512()
    {
        try
        {
            //      
            hashAlg  = X509ObjectIdentifiers.id_tc26_gost3411_12_512;
            sigAlg   = "GOST3411v12512WithGOST3410v12512";
            signKP   = KeyPairAndCertificateUtil.makeGOST3410v12512KeyPair();
            signCert = KeyPairAndCertificateUtil.makeCertificate(signKP, signDN, signKP, signDN, sigAlg);

            createSignAndVerifyIt();
        }
        catch(Exception ex)
        {
            log(ex);
            fail();
        }
    }

    protected void verifySignatureFile(String filename)
    {
        try
        {
            String test_data="1";

            Security.addProvider(new DIGTProvider());

            BufferedReader dataIn = new BufferedReader(
                    new InputStreamReader(
                            new FileInputStream(filename)));

            String buf = null;
            String sigData = "";
            while ((buf = dataIn.readLine()) != null)
            {
                sigData += buf;
            }

            byte[] sigBytes = Base64.decode(sigData);

            ByteArrayInputStream byteIn = new ByteArrayInputStream(sigBytes);
            ASN1InputStream      aIn = new ASN1InputStream(byteIn);

            CMSProcessable data = new CMSProcessableByteArray(test_data.getBytes("UTF-16LE"));
            // CMS   ASN 
            CMSSignedData sig = new CMSSignedData(data,
                    ContentInfo.getInstance(aIn.readObject()));

            //     data  
            //CMSSignedData sig = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));

            //     ,           
            CertStore certs = sig.getCertificatesAndCRLs("Collection", "DIGT");

            //    
            SignerInformationStore  signers = sig.getSignerInfos();
            Collection              c = signers.getSigners();
            Iterator                it = c.iterator();

            //   
            while (it.hasNext())
            {
                //    
                SignerInformation   signer = (SignerInformation)it.next();
                //     
                Collection          certCollection = certs.getCertificates(signer.getSID());
                Iterator        certIt = certCollection.iterator();
                X509Certificate cert = (X509Certificate)certIt.next();

                // .
                assertTrue(signer.verify(cert, "DIGT"));
            }

            dataIn.close();

        } catch (Exception ex) {
            fail(ex.getMessage());
        }
    }
    
    public void testPKCS7v01()
    {
        verifySignatureFile("test-01.sig");
    }

    public void testPKCS7v12256()
    {
        verifySignatureFile("test-12-256.sig");
    }

    public void testPKCS7v12512()
    {
        verifySignatureFile("test-12-512.sig");
    }
}
