/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.cms.test;

import com.digt.trusted.cms.*;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;
import com.digt.trusted.tsp.TSPHelper;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.*;
import java.security.KeyPair;
import java.security.Security;
import java.security.cert.CertStore;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 *   /       PKCS7
 */
public class SignedDataTestTSP
        extends TestCase
{
    /*
    *
    *  VARIABLES
    *
    */

    private static final int GOST3410 = 1;
    private static final int GOST3410EL = 2;
    private static final int GOSTDH = 3;
    private static final int GOSTECDH = 4;

    /**
     *     
     */
    private static final String tsaUrl = "http://www.cryptopro.ru/tsp/tsp.srf";

    boolean DEBUG = true;
    String          signDN;
    KeyPair         signKP;
    X509Certificate signCert;

    private static final String testDataFile = "test.txt";
    private static final String testSigFile = "test.sig";

    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public static void main(String args[]) {

        junit.textui.TestRunner.run(SignedDataTestTSP.class);
    }

    public static Test suite() {
        return new TestSuite(SignedDataTestTSP.class);
    }

    private void log(Exception _ex) {
        if(DEBUG) {
            _ex.printStackTrace();
        }
    }

    private void log(String _msg) {
        if(DEBUG) {
            System.out.println(_msg);
        }
    }

    protected void setUp()
    {
        try
        {
            Security.addProvider(new DIGTProvider());
        }
        catch(Exception ex)
        {
            //log(ex);
            fail(ex.getMessage());
        }
    }

    /**
     *       "GOST3411WithGOST3410EL"
     *           
     *   
     *
     */
    public void testPKCS7SignatureWithTSP()
    {
        try
        {
            signDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            signKP = makeKeyPair(GOST3410EL);
            signCert = KeyPairAndCertificateUtil.makeCertificate(signKP, signDN, signKP, signDN, "GOST3411WithGOST3410EL");

            ArrayList certList = new ArrayList();
            CMSProcessable msg = new CMSProcessableFile(new File(testDataFile));

            certList.add(signCert);

            CertStore certs = CertStore.getInstance("Collection",
                    new CollectionCertStoreParameters(certList), "DIGT");

            CMSSignedDataGenerator gen = new CMSSignedDataGenerator();

            TSPHelper tsp = new TSPHelper(msg);
            gen.addSigner(signKP.getPrivate(), signCert, CMSSignedDataGenerator.DIGEST_GOST3411,
                    TSPHelper.convertToAttr(tsp.getTimeStampOnline(tsaUrl)), null);

            gen.addCertificatesAndCRLs(certs);

            CMSSignedData s = gen.generate(msg, false, "DIGT");

            FileOutputStream sigOut = new FileOutputStream(testSigFile);
            sigOut.write(s.getEncoded());
            sigOut.close();

            FileInputStream sigIn = new FileInputStream(testSigFile);

            s = new CMSSignedData(msg, sigIn);

            certs = s.getCertificatesAndCRLs("Collection", "DIGT");

            SignerInformationStore  signers = s.getSignerInfos();
            Collection              c = signers.getSigners();
            Iterator                it = c.iterator();

            while (it.hasNext())
            {
                SignerInformation   signer = (SignerInformation)it.next();
                Collection          certCollection = certs.getCertificates(signer.getSID());

                Iterator        certIt = certCollection.iterator();
                X509Certificate cert = (X509Certificate)certIt.next();

                boolean result = signer.verify(cert, "DIGT");
                log("Signature verification status = " + result);
                assertEquals("Signature verification failed", true, result);
                tsp.verifySigner(signer);
                log("Timestamp verification status = ok");
            }
        }
        catch(Exception ex)
        {
            //log(ex);
            fail(ex.getMessage());
        }
    }

    private KeyPair makeKeyPair(int _algo) {
        switch (_algo) {
            case GOST3410:
                return KeyPairAndCertificateUtil.makeGOST3410KeyPair();
            case GOST3410EL:
                return KeyPairAndCertificateUtil.makeGOST3410ELKeyPair();
            case GOSTDH:
                return KeyPairAndCertificateUtil.makeGOSTDHKeyPair();
            case GOSTECDH:
                return KeyPairAndCertificateUtil.makeGOSTELDHKeyPair();
            default:
                return null;
        }
    }


}
