/*
 * Copyright(C) 1999-2008  << >>
 *
 *    , 
 *    << >>.
 *
 *        ,
 * ,    ,
 *     ,
 * ,     
 *     
 *     << >>.
 */
package com.digt.trusted.cms.test.bc;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.AlgorithmParameters;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.Provider;
import java.security.Provider.Service;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Set;

import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Set;
import org.bouncycastle.asn1.DERBMPString;
import org.bouncycastle.asn1.DEREncodable;
import org.bouncycastle.asn1.DERGeneralizedTime;
import org.bouncycastle.asn1.DERIA5String;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERPrintableString;
import org.bouncycastle.asn1.DERT61String;
import org.bouncycastle.asn1.DERUTCTime;
import org.bouncycastle.asn1.DERUTF8String;
import org.bouncycastle.asn1.DERVisibleString;
import org.bouncycastle.asn1.cms.Attribute;
import org.bouncycastle.asn1.cms.AttributeTable;
import org.bouncycastle.asn1.cms.ContentInfo;
import org.bouncycastle.cms.CMSEnvelopedData;
import org.bouncycastle.cms.RecipientId;
import org.bouncycastle.cms.RecipientInformation;
import org.bouncycastle.cms.RecipientInformationStore;


/*
 *        IDE
 */
public class EnvelopedDataAboutTest_bc {

    boolean DEBUG = true;
    String signDN;
    KeyPair signKP;
    X509Certificate signCert;

    String EncAlg="";
    String prov="";
    String alias=null;
    char[] pass = new char[0];
    byte[] in = null;

    public static void main(String[] args) {
//        Security.addProvider(new DIGTProvider());
        EnvelopedDataAboutTest_bc st = new EnvelopedDataAboutTest_bc();
        st.setUp();
        st.testPKCS7();
    }

    void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        signDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
    }

    /*
     *   
     *   
     *   OID
     */
    static String GetSimpleAttribute(AttributeTable table, String oid) {
        String str = null;
        if (table == null) {
            return str; //null
        }
        Attribute atr = table.get(new DERObjectIdentifier(oid));
        if (atr == null) {
            return str; //null
        }
        str = Attribute2String(atr);
        return str;
    }

    static String Attribute2String(Attribute attr) {
        String str = null;
        ASN1Set v = attr.getAttrValues();
        if (v.size() == 1) {
            DERObject obj = v.getObjectAt(0).getDERObject();
            str = DER2String(obj);
        }
        return str;
    }

    static String DER2String(DERObject obj) {
        String str = null;
        if (obj instanceof DERObjectIdentifier) {
            str = ((DERObjectIdentifier) obj).getId();
        } else if (obj instanceof DERBMPString) {
            str = ((DERBMPString) obj).getString();
        } else if (obj instanceof DERIA5String) {
            str = ((DERIA5String) obj).getString();
        } else if (obj instanceof DERUTF8String) {
            str = ((DERUTF8String) obj).getString();
        } else if (obj instanceof DERPrintableString) {
            str = ((DERPrintableString) obj).getString();
        } else if (obj instanceof DERVisibleString) {
            str = ((DERVisibleString) obj).getString();
        } else if (obj instanceof DERT61String) {
            str = ((DERT61String) obj).getString();

        } else if (obj instanceof DERUTCTime) {
            str = ((DERUTCTime) obj).getTime();
        } else if (obj instanceof DERGeneralizedTime) {
            str = ((DERGeneralizedTime) obj).getTime();
            //UTCTime(((DERUTCTime) obj).getTime());
        }
        return str;
    }

    void ASN1Set_About(Attribute attr, String prefix) {
        String dAs;
        dAs = Attribute2String(attr);
        if (dAs != null) {
            System.out.println(prefix + "AttributeTable_About: Value for oid (" + attr.getAttrType().getId() + ") = '" + dAs + "'");
        } else {
            ASN1Set v = attr.getAttrValues();
            System.out.println(prefix + "AttributeTable_About: Values for oid (" + attr.getAttrType().getId() + ") = ");
            for (int i = 0; i < v.size(); i++) {
                DERObject obj = v.getObjectAt(i).getDERObject();
                dAs = obj.toString();
                dAs = dAs.substring(0, Math.min(30, dAs.length())) + " ...";
                System.out.println(prefix + "AttributeTable_About: Value[" + (i + 1) + "] = " + dAs);
            }
        }
    }

    void AttributeTable_About(AttributeTable table, String prefix) {
        if (table == null) {
            System.out.println(prefix + "AttributeTable_About: None attributes!!!");
            return;
        }

        ASN1EncodableVector asn_table;
        asn_table = table.toASN1EncodableVector();
        int s = asn_table.size();
        for (int i = 0; i < s; i++) {
            DEREncodable dO = asn_table.get(i);
            Attribute atr = Attribute.getInstance(dO);
            ASN1Set_About(atr, prefix);
        }
    }

    private void CMSEnvelopedData_About(CMSEnvelopedData envData) {
        if (envData == null) return;
        EncAlg=envData.getEncryptionAlgOID();
        System.out.println("getEncryptionAlgOID() = " + EncAlg);

        Provider[] providers = Security.getProviders("Cipher." + EncAlg);
        Provider provider;
        String providerName = "";
        for (int i = 0; i < providers.length; i++) {
            provider = providers[i];
            providerName = provider.getName();
            String suffix = "Provider[" + i + "]=" + providerName;
            System.out.println("" + suffix);

            AlgorithmParameters encAlgParams = null;
            try {
                encAlgParams = envData.getEncryptionAlgorithmParameters(providerName);
                AlgorithmParameters_About(suffix + ": ", encAlgParams);
            } catch (Exception ex) {
                //ex.printStackTrace();
            }
            System.out.println("getEncryptionAlgorithmParameters = " + encAlgParams);

        }
        provider = providers[0];
        prov=provider.getName();

        //AttributeTable_About(envData.getUnprotectedAttributes(), "");
        ContentInfo_About(envData.getContentInfo(), "");
        RecipientInformationStore_About(envData.getRecipientInfos(),"");

    }

    private void AlgorithmParameters_About(String prefix, AlgorithmParameters Params) {

        System.out.println(prefix + "Params.getAlgorithm() = " + Params.getAlgorithm());
    }

    private void ContentInfo_About(ContentInfo contentInfo, String prefix) {
        System.out.println(prefix + "contentInfo.getContentType() = " + contentInfo.getContentType());
    }
    private void RecipientInformationStore_About(RecipientInformationStore recipientStore, String prefix) {
        System.out.println(prefix + "Recipients count = " + recipientStore.size());
        //    
        Collection c = recipientStore.getRecipients();
        Iterator it = c.iterator();

        //   
        while (it.hasNext()) {
            //    
            RecipientInformation recipient = (RecipientInformation) it.next();
            RecipientInformation_About(recipient, "About recipient: ");
        }
    }

    private void RecipientInformation_About(
            RecipientInformation recipient,
            String prefix) {
        String KeyEncAlg = recipient.getKeyEncryptionAlgOID();
        System.out.println(prefix + " getKeyEncryptionAlgOID() = " + KeyEncAlg);

        RecipientId rid = recipient.getRID();
        System.out.println(prefix + " getRID() = " + rid);

        //      
        Key sk = RecipientKeyInStore(prefix, rid);

        if (sk == null) {
//            System.out.println(prefix + " Recipient key not found in store!!!");
        } else {
            System.out.println(prefix + " Recipient key class = "+sk.getClass().getCanonicalName());
            System.out.println(prefix + " Key in store (alias) = " + this.alias);
            System.out.println(prefix + " key.getAlgorithm()  = " + sk.getAlgorithm());
            System.out.println(prefix + " JCE Provider  = " + this.prov);
            try {
                //Decrypting
                byte[] recData;
                if (sk.getClass().getCanonicalName().contains("digt.trusted")) {
                    recData = recipient.getGOSTContent(sk, this.prov,
                            new DataInputStream(new ByteArrayInputStream(this.in)));
                } else {
                    recData = recipient.getContent(sk, this.prov);
                }
                System.out.println(prefix + " after decrypting in recipient`s adress");
                System.out.println("========= Decrypted message =========");
                System.out.println(new String(recData));
                System.out.println("=====================================");
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private Key RecipientKeyInStore(String prefix, RecipientId rid){
        //      KeyStore
        System.out.println(prefix+"Search recipient`s key in all keystore");
        Key sk = null;
        KeyStore kstore = null;
        this.alias=null;
        Provider[] pr = Security.getProviders();
        for (int i = 0; i < pr.length; i++) {
            Iterator it_prs = ((Set) pr[i].getServices()).iterator();
            while (it_prs.hasNext()) {
                Service pr_s = (Service) it_prs.next();
                if (!pr_s.getType().equals("KeyStore")) {
                    continue;
                }

                System.out.println(prefix
                        + " Provider() = (" + pr[i].getName()
//                        + ") Service = (" + pr_s.getType()
                        + ") Algoritm = " + pr_s.getAlgorithm());
                try {
                    kstore = KeyStore.getInstance(pr_s.getAlgorithm(), pr[i].getName());
                    InputStream store_stream  = new ByteArrayInputStream("".getBytes("UTF-8"));
                    if (pr_s.getAlgorithm().equals("CryptoProCSPKeyStore")) {
                        store_stream = new ByteArrayInputStream("CurrentUser".getBytes("UTF-8"));
                    } else {//???   
                    }
                    kstore.load(store_stream, null);
                    //   
                    this.alias = RecipientKeyAlias(kstore, rid);
                    if (this.alias == null) {
                        System.out.println(prefix + " Recipient key not found in store!!!");
                        continue;
                    }
                    //   
                    this.prov=pr[i].getName();

                    //  -    
                    //this.pass = "11111111".toCharArray();

                    sk = kstore.getKey(this.alias, this.pass);
                    return sk;
                } catch (Exception ex) {
                        System.out.println(prefix + " Don`t implemented!!!");
                }
            }
        }
        return sk;
    }

    private static String RecipientKeyAlias(KeyStore kstore, RecipientId rid)
            throws KeyStoreException {
        String al = null;
        //      
        for (Enumeration aliases = kstore.aliases(); aliases.hasMoreElements();) {
            String alias_ = (String) aliases.nextElement();
            X509Certificate Cert = (X509Certificate) kstore.getCertificate(alias_);
            if (Cert.getIssuerX500Principal().equals(rid.getIssuer()) && Cert.getSerialNumber().equals(rid.getSerialNumber())) {
                al = alias_;
                break;
            }
        }
        return al;
    }

    public void testPKCS7() {

        //Security.addProvider(new DIGTProvider());

        CMSEnvelopedData envData = null;
        final String sfile = "data/text4enc.txt.enc";
        try {
            envData = new CMSEnvelopedData(new FileInputStream(sfile));
            this.in = envData.getEncoded();
            /*
             *         p7m.
            DataOutputStream out = new DataOutputStream( new BufferedOutputStream(new FileOutputStream("copy_text4enc.txt.enc")));
            for (int i = 0; i != this.in.length; i++)  out.writeByte(this.in[i]);
            out.close();
             */
        } catch (Exception ex) {
                ex.printStackTrace();
        }
        //    
        CMSEnvelopedData_About(envData);
    }

}
