/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.cms.test.bc;

import java.security.KeyPair;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.bouncycastle.asn1.cms.EnvelopedData;
import org.bouncycastle.cms.CMSEnvelopedData;
import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
import org.bouncycastle.cms.CMSProcessableByteArray;
import org.bouncycastle.cms.RecipientInformation;
import org.bouncycastle.cms.RecipientInformationStore;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;

public class EnvelopedDataTest_bc extends TestCase {

    /*
    *
    *  VARIABLES
    *
    */

    public boolean DEBUG = true;
    String          signDN;
    KeyPair         signKP;

    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public EnvelopedDataTest_bc(String name) {
        super(name);
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run(EnvelopedDataTest_bc.class);
    }

    public static Test suite() {
        return new TestSuite(EnvelopedDataTest_bc.class);
    }

    public void log(Exception _ex) {
        if(DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if(DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp()
    {
    }


    public void tearDown() {

    }

    /*
    *
    *  TESTS
    *
    */

    //   CMS    "GOST3410EL"    "GOST28147"
    //  ,   CSP    , , 
    //         Diffie-Hellman.
    //      "GOST3410EL"   34.11/34.10-2001
    public void testKeyTransGOST28147WithGOST3410EL()
    {
        try {
            Security.addProvider(new DIGTProvider());
            // 
            byte[]          data      = "Test Data".getBytes();

            //      ,      .
            //     AT_KEYEXCHANGE(MS CAPI),     
            //     "GOSTDH"  "GOSTECDH"   KeyPairGenerator.getInstance("GOSTDH", "DIGT")
            // ,     ,    
            String          _signDN   = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            String          _sigAlg   = "GOST3411WithGOST3410EL";
            KeyPair         _signKP   = KeyPairAndCertificateUtil.makeGOSTELDHKeyPair();
            X509Certificate _signCert = KeyPairAndCertificateUtil.makeCertificate(_signKP, _signDN, _signKP, _signDN,_sigAlg);

            //    ,      
            /*DataOutputStream out1 = new DataOutputStream( new BufferedOutputStream(new FileOutputStream("c:\\enc_cert.cer")));
            for (int i = 0; i != _signCert.getEncoded().length; i++) out1.writeByte(_signCert.getEncoded()[i]);
            out1.close();*/

            //   .
            CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
            // 
            edGen.addKeyTransRecipient(_signCert);

            /*
             * 
             *
            CMSEnvelopedData ed = edGen.generate(
            new CMSProcessableByteArray(data),
            CMSEnvelopedDataGenerator.GOST28147_CFB, "DIGT");
             */
            CMSEnvelopedData ed = edGen.generate(
                    new CMSProcessableByteArray(data),
                    "1.2.643.2.2.21",
                    "DIGT");

            //      
            //byte[] ec = ((EnvelopedData)ed.getcontentInfo().getContent()).
            //        getEncryptedContentInfo().getGOSTEncryptedContent();
            byte[] ec = ((EnvelopedData)ed.getContentInfo().getContent()).
                    getEncryptedContentInfo().getEncryptedContent().getOctets();
            //      p7m.
            /*
            DataOutputStream out = new DataOutputStream(
                    new BufferedOutputStream(
                        new FileOutputStream("gost_enveloped_bc.p7m")
                        )
                    );
            for (int i = 0; i != ec.length; i++)  out.writeByte(ec[i]);
            out.close();
             */

            CMSEnvelopedData envData = new CMSEnvelopedData(new ByteArrayInputStream(ec));
            ec = envData.getEncoded();
            //  
            DataInputStream in = new DataInputStream( new ByteArrayInputStream(ec));
            //  
            RecipientInformationStore  recipients = ed.getRecipientInfos();
            Collection  c = recipients.getRecipients();
            Iterator    it = c.iterator();

            while (it.hasNext())
            {
                //           
                //    ,      .
                //        .
                RecipientInformation   recipient = (RecipientInformation)it.next();
                //  

                byte[] recData = recipient.getGOSTContent(_signKP.getPrivate(), "DIGT", in);
                //,       
                log("before encrypting = " + new String(data) + " after decrypting = " + new String(recData));
                assertEquals(true, Arrays.equals(data, recData));
            }
        }
        catch(Exception ex) {
            log(ex);
            fail();
        }
    }
}
