/*
 * Copyright(C) 1999-2008  << >>
 *
 *    , 
 *    << >>.
 *
 *        ,
 * ,    ,
 *     ,
 * ,     
 *     
 *     << >>.
 */
package com.digt.trusted.cms.test.bc;

import java.io.ByteArrayInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.security.KeyPair;
import java.security.Provider;
import java.security.Security;
import java.security.cert.CertStore;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1Set;
import org.bouncycastle.asn1.DERBMPString;
import org.bouncycastle.asn1.DEREncodable;
import org.bouncycastle.asn1.DERGeneralizedTime;
import org.bouncycastle.asn1.DERIA5String;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERPrintableString;
import org.bouncycastle.asn1.DERT61String;
import org.bouncycastle.asn1.DERUTCTime;
import org.bouncycastle.asn1.DERUTF8String;
import org.bouncycastle.asn1.DERVisibleString;
import org.bouncycastle.asn1.cms.Attribute;
import org.bouncycastle.asn1.cms.ContentInfo;
import org.bouncycastle.asn1.cms.AttributeTable;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;

import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;

import org.bouncycastle.util.encoders.Base64;

/*
 *        IDE
 */
public class SignedDataAttributesTest_bc {

    boolean DEBUG = true;
    String signDN;
    KeyPair signKP;
    X509Certificate signCert;

    public static void main(String[] args) {
//        Security.addProvider(new DIGTProvider());
        SignedDataAttributesTest_bc st = new SignedDataAttributesTest_bc();
        st.setUp();
        st.testPKCS7();
    }

    void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        signDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
    }

    /*
     *    
     */
    static String GetCreateTime(AttributeTable table) {
        return GetSimpleAttribute(table, "1.2.840.113549.1.9.5");
    }

    /*
     *    
     */
    static String GetComment(AttributeTable table) {
        final String oid = "1.2.643.2.45.1.1.1";
        final String oid_old = "1.3.6.1.4.1.311.2.1.120";
        String str = GetSimpleAttribute(table, oid);
        if (str != null) {
            return str; //null
        }
        /*
         *    oid
         */
        str = GetSimpleAttribute(table, oid_old, 0);
        return str;
    }

    /*
     *    
     */
    static String GetResourceId(AttributeTable table) {
        final String oid = "1.2.643.2.45.1.1.2";
        final String oid_old = "1.3.6.1.4.1.311.2.1.120";
        String str = GetSimpleAttribute(table, oid);
        if (str != null) {
            return str; //null
        }
        /*
         *    oid
         */
        str = GetSimpleAttribute(table, oid_old, 1);
        return str;
    }

    /*
     *    
     */
    static String GetUsingOID(AttributeTable table) {
        final String oid = "1.2.643.2.45.1.1.3";
        final String oid_old = "1.3.6.1.4.1.311.2.1.120";
        String str = GetSimpleAttribute(table, oid);
        if (str != null) {
            return str; //null
        }
        /*
         *    oid
         */
        str = GetSimpleAttribute(table, oid_old, 2);
        return str;
    }

    /*
     *   
     *   
     *   OID
     */
    static String GetSimpleAttribute(AttributeTable table, String oid) {
        String str = null;
        if (table == null) {
            return str; //null
        }
        Attribute atr = table.get(new DERObjectIdentifier(oid));
        if (atr == null) {
            return str; //null
        }
        str = Attribute2String(atr);
        return str;
    }

    /*
     *   OID  
     * i-    
     */
    static String GetSimpleAttribute(AttributeTable table, String oid, int index) {
        String str = null;
        if (table == null) {
            return str; //null
        }
        Attribute atr = table.get(new DERObjectIdentifier(oid));
        if (atr == null) {
            return str; //null
        }
        ASN1Set v = atr.getAttrValues();
        DERObject vv = v.getObjectAt(0).getDERObject();
        Enumeration e = ((ASN1Sequence) vv).getObjects();
        int i = 0;
        while (e.hasMoreElements()) {
            Object o = e.nextElement();
            i++;
            if (i == index + 1) {
                str = DER2String((DERObject) o);
                return str;
            }
        }
        return str;//null
    }

    static String Attribute2String(Attribute attr) {
        String str = null;
        ASN1Set v = attr.getAttrValues();
        if (v.size() == 1) {
            DERObject obj = v.getObjectAt(0).getDERObject();
            str = DER2String(obj);
        }
        return str;
    }

    static String DER2String(DERObject obj) {
        String str = null;
        if (obj instanceof DERObjectIdentifier) {
            str = ((DERObjectIdentifier) obj).getId();
        } else if (obj instanceof DERBMPString) {
            str = ((DERBMPString) obj).getString();
        } else if (obj instanceof DERIA5String) {
            str = ((DERIA5String) obj).getString();
        } else if (obj instanceof DERUTF8String) {
            str = ((DERUTF8String) obj).getString();
        } else if (obj instanceof DERPrintableString) {
            str = ((DERPrintableString) obj).getString();
        } else if (obj instanceof DERVisibleString) {
            str = ((DERVisibleString) obj).getString();
        } else if (obj instanceof DERT61String) {
            str = ((DERT61String) obj).getString();

        } else if (obj instanceof DERUTCTime) {
            str = ((DERUTCTime) obj).getTime();
        } else if (obj instanceof DERGeneralizedTime) {
            str = ((DERGeneralizedTime) obj).getTime();
            //UTCTime(((DERUTCTime) obj).getTime());
        }
        return str;
    }

    void ASN1Set_About(Attribute attr, String prefix) {
        String dAs;
        String prefix2 = "AttributeTable_About: ";
        dAs = Attribute2String(attr);
        if (dAs != null) {
            System.out.println(prefix + prefix2 + "Value for oid (" + attr.getAttrType().getId() + ") = '" + dAs + "'");
        } else {
            ASN1Set v = attr.getAttrValues();
            System.out.println(prefix + prefix2 + "Values for oid (" + attr.getAttrType().getId() + ") = ");
            for (int i = 0; i < v.size(); i++) {
                DERObject obj = v.getObjectAt(i).getDERObject();
                dAs = obj.toString();
                dAs = dAs.substring(0, Math.min(30, dAs.length())) + " ...";
                System.out.println(prefix + prefix2 + "Value[" + (i + 1) + "] = " + dAs);
            }
        }
    }

    void AttributeTable_About(AttributeTable table, String prefix) {
        if (table == null) {
            System.out.println(prefix + "AttributeTable_About: None attributes!!!");
            return;
        }

        ASN1EncodableVector asn_table;
        asn_table = table.toASN1EncodableVector();
        int s = asn_table.size();
        for (int i = 0; i < s; i++) {
            DEREncodable dO = asn_table.get(i);
            Attribute atr = Attribute.getInstance(dO);
            ASN1Set_About(atr, prefix);
        }
    }

    Provider SignerInformatiom_About(CMSSignedData sig, SignerInformation signer, String prefix) {
        String prefix2 = "Signer_About: ";
        String prefix3 = "SignedAttribute_About: ";
        System.out.println(prefix + prefix2 + "Version = " + signer.getVersion());
        System.out.println(prefix + prefix2 + "Issuer  = " + signer.getSID().getIssuerAsString());
        System.out.println(prefix + prefix2 + "Digest Algoritm OID =" + signer.getDigestAlgOID());
        System.out.println(prefix + prefix2 + "Encryption Algoritm OID =" + signer.getEncryptionAlgOID());
        String signOid = signer.getDigestAlgOID() + "with" + signer.getEncryptionAlgOID();
        System.out.println(prefix + prefix2 + "Signature Algoritm = " + signOid);

        //MessageDigest md = MessageDigest.getInstance(signer.getDigestAlgOID());
        //Provider provider = md.getProvider(); //error

        //Provider[] providers = Security.getProviders("MessageDigest." + signer.getDigestAlgOID());
        Provider[] providers = Security.getProviders("Signature." + signOid);
        Provider provider;
        for (int i = 0; i < providers.length; i++) {
            provider = providers[i];
            String providerName = provider.getName();
            System.out.println(prefix + prefix2 + "Provider[i]=" + providerName);
        }
        provider = providers[0];
        String providerName = provider.getName();

        AttributeTable u_table = signer.getUnsignedAttributes();
        System.out.println(prefix + prefix2 + "UnsignedAttributes ----------- ");
        AttributeTable_About(u_table, prefix);
        AttributeTable s_table = signer.getSignedAttributes();
        System.out.println(prefix + prefix2 + "SignedAttributes ----------- ");
        AttributeTable_About(s_table, prefix);
        System.out.println(prefix + prefix2 + "SignedAttributes End ------- ");

        System.out.println(prefix + prefix3 + "TD_Sign.GetCreateTime(s_table) = '"
                + GetCreateTime(s_table) + "'");

        System.out.println(prefix + prefix3 + "TD_Sign.GetComment(s_table) = '"
                + GetComment(s_table) + "'");
        System.out.println(prefix + prefix3 + "TD_Sign.GetResourceId(s_table) = '"
                + GetResourceId(s_table) + "'");
        System.out.println(prefix + prefix3 + "TD_Sign.GetUsingOID(s_table) = '"
                + GetUsingOID(s_table) + "'");

        X509Certificate cert=null;
        try {
            //     ,           
            CertStore certs = sig.getCertificatesAndCRLs("Collection", providerName);
            //     
            Collection certCollection = certs.getCertificates(signer.getSID());
            Iterator certIt = certCollection.iterator();
            cert = (X509Certificate) certIt.next();

            /*
             *  .
             */
            System.out.println(prefix + prefix2 + "Subject  = " + cert.getSubjectDN());
            if (signer.verify(cert, providerName)) {
                System.out.println(prefix + prefix2 + "Sign verified!!");
            }
        } catch (Exception ex) {
            System.out.println(prefix + prefix2 + "Sign not verified: " + ex.getLocalizedMessage()+ "!!!");
            //ex.printStackTrace();
        }

        SignerInformationStore co_signer = signer.getCounterSignatures();
        int size = co_signer.size();
        System.out.println(prefix + prefix2 + "co-signers count = " + size);
        System.out.println(prefix + "=========");
        if (size > 0) {
            //     
            SignerInformationStore_About(sig, co_signer, prefix + " Co-signers : ");
        }

        return provider;
    }

    void SignerInformationStore_About(CMSSignedData sig, SignerInformationStore signers, String prefix) {

        System.out.println(prefix + "SignerStore_About: Count signers = " + signers.size());

        //    
        Collection c = signers.getSigners();
        Iterator it = c.iterator();

        //   
        while (it.hasNext()) {
            //    
            SignerInformation signer = (SignerInformation) it.next();
            SignerInformatiom_About(sig, signer, prefix);
        }
    }

    void CMSSignedData_About(CMSSignedData sig, String prefix) {
        //   
        System.out.println(prefix + "Version = " + sig.getVersion());
        System.out.println(prefix + "Signed Content Type OID= " + sig.getSignedContentTypeOID());
    }

    private String getStringFromFile(String inFile) {
        String s = "";
        try {
            BufferedReader dataIn = new BufferedReader(
                    new InputStreamReader(new FileInputStream(inFile)));

            String buf = null;
            while ((buf = dataIn.readLine()) != null) {
                s += buf;
            }
            dataIn.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return s;
    }

    private byte[] getBytesFromFile(String pFile) {
        int sl = 0,
                count = 0;
        byte[] b = new byte[0];
        try {
            FileInputStream inFile = new FileInputStream(pFile);
            // ,     
            int bytesAvailable = 0;
            while ((bytesAvailable = inFile.available()) != 0) {
                //   
                b = Arrays.copyOf(b, bytesAvailable + sl);
                count = inFile.read(b, sl, bytesAvailable);
                sl += count;
            }
            inFile.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return b;
    }

    public void testPKCS7() {

        //Security.addProvider(new DIGTProvider());

        byte[] sigBytes;
        //String sigData = getStringFromFile("data/attrs_test.txt.b64.sig");
        //byte[] sigBytes = Base64.decode(sigData);

        byte [] sigData = getBytesFromFile("data/attrs_test.txt.b64.sig");
        if (sigData[0] == 48) {
            //DER-encoding
            sigBytes = sigData;
        } else {
            //Base64-encoding
            sigBytes = Base64.decode(sigData);
        }

        ByteArrayInputStream byteIn = new ByteArrayInputStream(sigBytes);
        ASN1InputStream aIn = new ASN1InputStream(byteIn);

        // Defime ContentType of File
        ContentInfo ci = null;
        try {
            ci = ContentInfo.getInstance(aIn.readObject());
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        System.out.print("File_About: ContentInfo Type = " + ci.getContentType().getId());

        if (ci.getContentType().equals(PKCSObjectIdentifiers.signedData)) {
            System.out.println(" (signedData)");

            //CMSProcessable data = new CMSProcessableByteArray(test_data.getBytes("UTF-16LE"));
            // CMS   ASN 
            //sig = new CMSSignedData(data,ci);
            //     data  
            CMSSignedData sig = new CMSSignedData(ci);
            CMSSignedData_About(sig,"CMSSignedData_About: ");

            //     
            SignerInformationStore_About(sig, sig.getSignerInfos(), " Signers : ");
        }

    }
}
