/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.jce.provider.test;

import com.digt.trusted.util.encoders.Base64;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;

import java.security.*;
import java.security.cert.X509Certificate;
import java.io.ByteArrayInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.lang.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class CryptoProCSPKeyStoreTest
        extends TestCase {
    /*
    *
    *  VARIABLES
    *
    */

    public boolean DEBUG = true;
    String certDN;
    KeyPair certKP;
    X509Certificate cert;
    byte[] req;


    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public CryptoProCSPKeyStoreTest(String name) {
        super(name);
    }

    public static void main(String args[]) {

        junit.textui.TestRunner.run(CryptoProCSPKeyStoreTest.class);
    }

    public static junit.framework.Test suite() {
        return new TestSuite(CryptoProCSPKeyStoreTest.class);
    }

    public void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        Security.addProvider(new DIGTProvider());
    }

    public void tearDown() {
    }

    /*
    *
    *  TESTS
    *
    */
    public void testCertificateRequest() {
      try {
          certDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            certKP = KeyPairAndCertificateUtil.makeGOST3410ELKeyPair();
            req = KeyPairAndCertificateUtil.makeCertificateRequest(certKP, certDN, "GOST3411WithGOST3410EL");

            //write request to file (encode base64).
            byte[] pem_req = Base64.encode(req);
            FileOutputStream fos = new FileOutputStream(System.getProperty("user.home") + System.getProperty("file.separator") + "req.csr");
            fos.write(pem_req);
            fos.close();
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testLoadKeyStore() {
        try {
            KeyStore store = KeyStore.getInstance("CryptoProCSPKeyStore", "DIGT");
            
            store.load(new ByteArrayInputStream("CurrentUser".getBytes("UTF-8")), null); // default is private store ("my")
            //store.load(new ByteArrayInputStream("CurrentUser/Containers".getBytes("UTF-8")), null); // connected private key containers
            //store.load(new ByteArrayInputStream("LocalComputer/root".getBytes("UTF-8")), null); // machine root store ("Root")
            for (Enumeration aliases = store.aliases(); aliases.hasMoreElements();)
            {
                String alias = (String) aliases.nextElement();
                StringBuffer _sb = new StringBuffer();
                _sb.append("AliasName =  ");
                _sb.append(alias);
                _sb.append(" is KeyEntry ");
                _sb.append(store.isKeyEntry(alias));
                _sb.append(" is CertificateEntry ");
                _sb.append(store.isCertificateEntry(alias));
                log(_sb.toString());

                X509Certificate cert = (X509Certificate) store.getCertificate(alias);
                if (cert!= null)
                {
                    log( "Certificate SerialNumber = "+cert.getSerialNumber().toString() +": DN = "+ cert.getSubjectDN().toString() );
             //     log(cert.toString());
                }
                else 
                {
                    log( "Certificate is null"); 
                }
            }

        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }
}
