/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.jce.provider.test;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.util.encoders.Hex;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.security.*;
import java.security.Key;

/**
 * basic test class for the GOST28147 cipher
 */
public class GOST28147Test
        extends TestCase {

    /*
    *
    *  VARIABLES
    *
    */
    boolean DEBUG = true;
    static String[] cipherTests =
            {
                // ECB
                "256",
                "0123456789abcdef",
                "4e6f77206973207468652074696d6520666f7220616c6c20",
                "8ad3c8f56b27ff1fbd46409359bdc796bc350e71aac5f5c0",

                // CBC
                "256",
                "0123456789abcdef",
                "4e6f77206973207468652074696d6520666f7220616c6c20",
                "8fc87960c5cfafaf2652783546d42768ac5378901483a78e",

                // CFB
                "256",
                "0123456789abcdef",
                "4e6f77206973207468652074696d6520666f7220616c6c2000",
                "eae4a25b024737aa727cf0f36eb4d79e3cc56228d43b018c5f",

                // OFB (CNT)
                "256",
                "0123456789abcdef",
                "4e6f77206973207468652074696d6520666f7220616c6c2000",
                "573b1f0ffc11d2a179686707b457595379e051110d24004353"
            };

    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public GOST28147Test(String name) {
        super(name);
    }

    public static void main(String args[]) {

        junit.textui.TestRunner.run(GOST28147Test.class);
    }

    public static junit.framework.Test suite() {
        return new TestSuite(GOST28147Test.class);
    }

    public void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        Security.addProvider(new DIGTProvider());
    }

    public void tearDown() {
    }

    /*
    *
    *  TESTS
    *
    */

    public void testGOST28147() {

        for (int i = 0; i < cipherTests.length; ) {
            GOST28147ECB(Integer.parseInt(cipherTests[i]),
                    Hex.decode(cipherTests[i + 1]),
                    Hex.decode(cipherTests[i + 2]),
                    Hex.decode(cipherTests[i + 3]));
            i += 4;

            GOST28147CBC(Integer.parseInt(cipherTests[i]),
                    Hex.decode(cipherTests[i + 1]),
                    Hex.decode(cipherTests[i + 2]),
                    Hex.decode(cipherTests[i + 3]));
            i += 4;

            GOST28147CFB(Integer.parseInt(cipherTests[i]),
                    Hex.decode(cipherTests[i + 1]),
                    Hex.decode(cipherTests[i + 2]),
                    Hex.decode(cipherTests[i + 3]));
            i += 4;

            GOST28147OFB(Integer.parseInt(cipherTests[i]),
                    Hex.decode(cipherTests[i + 1]),
                    Hex.decode(cipherTests[i + 2]),
                    Hex.decode(cipherTests[i + 3]));
            i += 4;
        }
    }

    /*
    *
    *  INTERNAL METHODS
    *
    */
    private void GOST28147ECB(int strength,
                              byte[] keyBytes,
                              byte[] input,
                              byte[] output) {
        Key key;
        Cipher in, out;
        CipherInputStream cIn;
        CipherOutputStream cOut;
        ByteArrayInputStream bIn;
        ByteArrayOutputStream bOut;

        try {
            key = new SecretKeySpec(keyBytes, "GOST28147");

            in = Cipher.getInstance("GOST28147/ECB/NoPadding", "DIGT");
            out = Cipher.getInstance("GOST28147/ECB/NoPadding", "DIGT");
            out.init(Cipher.ENCRYPT_MODE, key);
            in.init(Cipher.DECRYPT_MODE, key);

            //
            // encryption
            //
            bOut = new ByteArrayOutputStream();

            cOut = new CipherOutputStream(bOut, out);
            for (int i = 0; i != input.length / 2; i++) {
                cOut.write(input[i]);
            }
            cOut.write(input, input.length / 2, input.length - input.length / 2);
            cOut.close();

            byte[] bytes;

            bytes = bOut.toByteArray();

            log("GOST28147 ECB encryption test - expected " + new String(Hex.encode(output)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, output));

            //
            // decryption
            //
            bIn = new ByteArrayInputStream(bytes);

            cIn = new CipherInputStream(bIn, in);
            DataInputStream dIn = new DataInputStream(cIn);

            bytes = new byte[output.length];

            for (int i = 0; i != output.length / 2; i++) {
                bytes[i] = (byte) dIn.read();
            }
            dIn.readFully(bytes, output.length / 2, bytes.length - output.length / 2);

            log("GOST28147 ECB decryption test - expected " + new String(Hex.encode(input)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, input));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    private void GOST28147CBC(int strength,
                              byte[] keyBytes,
                              byte[] input,
                              byte[] output) {
        Key key;
        Cipher in, out;
        CipherInputStream cIn;
        CipherOutputStream cOut;
        ByteArrayInputStream bIn;
        ByteArrayOutputStream bOut;

        try {
            key = new SecretKeySpec(keyBytes, "GOST28147");

            in = Cipher.getInstance("GOST28147/CBC/NoPadding", "DIGT");
            out = Cipher.getInstance("GOST28147/CBC/NoPadding", "DIGT");
            byte[] iv = {1, 2, 3, 4, 5, 6, 7, 8};
            out.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(iv));
            in.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(iv));

            //
            // encryption
            //
            bOut = new ByteArrayOutputStream();

            cOut = new CipherOutputStream(bOut, out);
            for (int i = 0; i != input.length / 2; i++) {
                cOut.write(input[i]);
            }
            cOut.write(input, input.length / 2, input.length - input.length / 2);
            cOut.close();

            byte[] bytes;

            bytes = bOut.toByteArray();

            log("GOST28147 CBC encryption test - expected " + new String(Hex.encode(output)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, output));

            //
            // decryption
            //
            bIn = new ByteArrayInputStream(bytes);

            cIn = new CipherInputStream(bIn, in);
            DataInputStream dIn = new DataInputStream(cIn);

            bytes = new byte[output.length];

            for (int i = 0; i != output.length / 2; i++) {
                bytes[i] = (byte) dIn.read();
            }
            dIn.readFully(bytes, output.length / 2, bytes.length - output.length / 2);

            log("GOST28147 CBC decryption test - expected " + new String(Hex.encode(input)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, input));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    private void GOST28147CFB(int strength,
                              byte[] keyBytes,
                              byte[] input,
                              byte[] output) {
        Key key;
        Cipher in, out;
        CipherInputStream cIn;
        CipherOutputStream cOut;
        ByteArrayInputStream bIn;
        ByteArrayOutputStream bOut;

        try {
            key = new SecretKeySpec(keyBytes, "GOST28147");

            in = Cipher.getInstance("GOST28147/CFB/NoPadding", "DIGT"); // just CFB means default size of internal buffer
            out = Cipher.getInstance("GOST28147/CFB8/NoPadding", "DIGT"); // CFB8 means size of internal buffer = 1 byte
            byte[] iv = {1, 2, 3, 4, 5, 6, 7, 8};
            out.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(iv));
            in.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(iv));

            //
            // encryption
            //
            bOut = new ByteArrayOutputStream();

            cOut = new CipherOutputStream(bOut, out);

            for (int i = 0; i != input.length / 2; i++) {
                cOut.write(input[i]);
            }
            cOut.write(input, input.length / 2, input.length - input.length / 2);
            cOut.close();
            byte[] bytes;

            bytes = bOut.toByteArray();

            log("GOST28147 CFB decryption test - expected " + new String(Hex.encode(output)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, output));


            //
            // decryption
            //
            bIn = new ByteArrayInputStream(bytes);

            cIn = new CipherInputStream(bIn, in);
            DataInputStream dIn = new DataInputStream(cIn);

            bytes = new byte[output.length];

            for (int i = 0; i != output.length / 2; i++) {
                bytes[i] = (byte) dIn.read();
            }
            dIn.readFully(bytes, output.length / 2, bytes.length - output.length / 2);

            log("GOST28147 CFB decryption test - expected " + new String(Hex.encode(input)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, input));

        } catch (Exception ex) {
            log(ex);
            fail();

        }
    }

    private void GOST28147OFB(int strength,
                              byte[] keyBytes,
                              byte[] input,
                              byte[] output) {
        Key key;
        Cipher in, out;
        CipherInputStream cIn;
        CipherOutputStream cOut;
        ByteArrayInputStream bIn;
        ByteArrayOutputStream bOut;

        try {
            key = new SecretKeySpec(keyBytes, "GOST28147");

            in = Cipher.getInstance("GOST28147/OFB8192/NoPadding", "DIGT"); // OFB8 means size of internal buffer = 1024 bytes
            out = Cipher.getInstance("GOST28147/CNT/NoPadding", "DIGT"); // CNT is equivalent OFB
            byte[] iv = {1, 2, 3, 4, 5, 6, 7, 8};
            out.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(iv));
            in.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(iv));

            //
            // encryption
            //
            bOut = new ByteArrayOutputStream();

            cOut = new CipherOutputStream(bOut, out);

            for (int i = 0; i != input.length / 2; i++) {
                cOut.write(input[i]);
            }
            cOut.write(input, input.length / 2, input.length - input.length / 2);
            cOut.close();
            byte[] bytes;

            bytes = bOut.toByteArray();

            log("GOST28147 OFB decryption test - expected " + new String(Hex.encode(output)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, output));


            //
            // decryption
            //
            bIn = new ByteArrayInputStream(bytes);

            cIn = new CipherInputStream(bIn, in);
            DataInputStream dIn = new DataInputStream(cIn);

            bytes = new byte[output.length];

            for (int i = 0; i != output.length / 2; i++) {
                bytes[i] = (byte) dIn.read();
            }
            dIn.readFully(bytes, output.length / 2, bytes.length - output.length / 2);

            log("GOST28147 OFB decryption test - expected " + new String(Hex.encode(input)) + " got " + new String(Hex.encode(bytes)));
            assertEquals(true, equalArray(bytes, input));

        } catch (Exception ex) {
            log(ex);
            fail();

        }
    }

    private boolean equalArray(byte[] a,
                               byte[] b) {
        if (a.length != b.length) {
            return false;
        }

        for (int i = 0; i != a.length; i++) {
            if (a[i] != b[i]) {
                return false;
            }
        }

        return true;
    }


}