/*
* Copyright(C) 1999-2015  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.jce.provider.test;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;

import java.security.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class GOST3410v12Test
        extends TestCase {
    boolean DEBUG = true;

    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public GOST3410v12Test(String name) {
        super(name);
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run(GOST3410v12Test.class);
    }

    public static junit.framework.Test suite() {
        return new TestSuite(GOST3410v12Test.class);
    }

    public void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        Security.addProvider(new DIGTProvider());
    }

    public void tearDown() {
    }

    /*
    *
    *  TESTS
    *
    */
    public void testGOST3410v12_256() {
        try {
            boolean status = false;
            String data = "Test Data";
            Signature sig = Signature.getInstance("GOST3410v12256", "DIGT");
            KeyPair signKP = KeyPairAndCertificateUtil.makeGOST3410v12256KeyPair();
            PrivateKey privKey = signKP.getPrivate();
            PublicKey pubKey = signKP.getPublic();

            sig.initSign(privKey);
            sig.update(data.getBytes());
            byte[] sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12256", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

            sig = Signature.getInstance("GOST3410v12256", "DIGT");
            sig.initSign(privKey);
            sig.update(data.getBytes());
            sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12256", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testGOSTv12256DH() {
        try {
            boolean status = false;
            String data = "Test Data";
            Signature sig = Signature.getInstance("GOST3410v12256", "DIGT");
            KeyPair signKP = KeyPairAndCertificateUtil.makeGOSTv12256DHKeyPair();
            PrivateKey privKey = signKP.getPrivate();
            PublicKey pubKey = signKP.getPublic();

            sig.initSign(privKey);
            sig.update(data.getBytes());
            byte[] sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12256", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

            sig = Signature.getInstance("GOST3410v12256", "DIGT");
            sig.initSign(privKey);
            sig.update(data.getBytes());
            sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12256", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testGOST3410v12_512() {
        try {
            boolean status = false;
            String data = "Test Data";
            Signature sig = Signature.getInstance("GOST3410v12512", "DIGT");
            KeyPair signKP = KeyPairAndCertificateUtil.makeGOST3410v12512KeyPair();
            PrivateKey privKey = signKP.getPrivate();
            PublicKey pubKey = signKP.getPublic();

            sig.initSign(privKey);
            sig.update(data.getBytes());
            byte[] sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12512", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

            sig = Signature.getInstance("GOST3410v12512", "DIGT");
            sig.initSign(privKey);
            sig.update(data.getBytes());
            sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12512", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testGOSTv12512DH() {
        try {
            boolean status = false;
            String data = "Test Data";
            Signature sig = Signature.getInstance("GOST3410v12512", "DIGT");
            KeyPair signKP = KeyPairAndCertificateUtil.makeGOSTv12512DHKeyPair();
            PrivateKey privKey = signKP.getPrivate();
            PublicKey pubKey = signKP.getPublic();

            sig.initSign(privKey);
            sig.update(data.getBytes());
            byte[] sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12512", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

            sig = Signature.getInstance("GOST3410v12512", "DIGT");
            sig.initSign(privKey);
            sig.update(data.getBytes());
            sigBytes = sig.sign();

            sig = Signature.getInstance("GOST3410v12512", "DIGT");
            sig.initVerify(pubKey);
            sig.update(data.getBytes());
            status = sig.verify(sigBytes);

            log("Signature verification status = " + status);
            assertEquals(true, status);

        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }
}
