/*
* Copyright(C) 1999-2015  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.jce.provider.test;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;
import com.digt.trusted.util.encoders.Hex;

import java.security.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class GOST3411v12Test
        extends TestCase
{
    /*
    *
    *  VARIABLES
    *
    */
    boolean DEBUG = true;

    static String[] hashesTests =
            {
                // 256 bit (data and hash value are taken from GOST R 34.11-2012, appendix A)
                "303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132", // 63 bytes of data
                "9d151eefd8590b89daa6ba6cb74af9275dd051026bb149a452fd84e5e57b5500", // 32 bytes of hash value

                // 512 bit (data and hash value are taken from GOST R 34.11-2012, appendix A)
                "303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132", // 63 bytes of data
                "1b54d01a4af5b9d5cc3d86d68d285462b19abc2475222f35c085122be4ba1ffa00ad30f8767b3a82384c6574f024c311e2a481332b08ef7f41797891c1646f48" // 64 bytes of hash value
            };
    
    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public GOST3411v12Test(String name) {
        super(name);
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run(GOST3411v12Test.class);
    }

    public static junit.framework.Test suite() {
        return new TestSuite(GOST3411v12Test.class);
    }

    public void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
        Security.addProvider(new DIGTProvider());
    }

    public void tearDown() {
    }

    /*
    *
    *  TESTS
    *
    */

    public void testGOST3411_2012_256bit() {
        try {
            MessageDigest md = MessageDigest.getInstance("GOST3411v12256");

            md.update(Hex.decode(hashesTests[0]));
            byte[] digest = md.digest();

            log("GOST R 34.11-2012 (256 bit) test - expected " + hashesTests[1] + " got " + new String(Hex.encode(digest)));
            assertEquals(true, MessageDigest.isEqual(digest, Hex.decode(hashesTests[1])));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testGOST3411_2012_512bit() {
        try {
            MessageDigest md = MessageDigest.getInstance("GOST3411v12512");

            md.update(Hex.decode(hashesTests[2]));
            byte[] digest = md.digest();
            
            log("GOST R 34.11-2012 (512 bit) test - expected " + hashesTests[3] + " got " + new String(Hex.encode(digest)));
            assertEquals(true, MessageDigest.isEqual(digest, Hex.decode(hashesTests[3])));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }
}
