/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.jce.provider.test;

import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.cert.X509Certificate;
import java.util.Calendar;

/**
 * User: andy
 * Date: 06.06.2005
 * Time: 11:14:33
 */
public class PerformanceTest
        extends TestCase {

    /*
    *
    *  VARIABLES
    *
    */
    boolean DEBUG = true;
    static final int testsCount = 2;
    static final int threadCount = 20;
    static final int signLen = 64;
    static final int GOST3410 = 1;
    static final int GOST3410EL = 2;
    static final int GOSTDH = 3;
    static final int GOSTECDH = 4;
    private static int threads = 0;

    /*
    *
    *  INFRASTRUCTURE
    *
    */

    public PerformanceTest(String name) {
        super(name);
    }

    public static void main(String args[]) {

        junit.textui.TestRunner.run(PerformanceTest.class);
    }

    public static junit.framework.Test suite() {
        return new TestSuite(PerformanceTest.class);
    }

    public void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }

    public void setUp() {
    }

    public void tearDown() {
    }

    /*
    *
    *  TESTS
    *
    */

    public void testSignVerify() {

        String _data = "Test data";
        byte[] _sigBytes = new byte[signLen];

        KeyPair _signKP = null;
        long _start = 0;

        //_signKP = makeKeyPair(GOST3410);
        //_start = getTime();
        //for (int i = 0; i < testsCount; i++) {
        //    _sigBytes = Sign(GOST3410, _data, _signKP);
        //    Verify(GOST3410, _data, _signKP, _sigBytes);
        //}
        //log(runTime("Sign and Verify Test for GOST3410 ", _start));

        _signKP = makeKeyPair(GOST3410EL);
        _start = getTime();
        for (int i = 0; i < testsCount; i++) {
            _sigBytes = Sign(GOST3410EL, _data, _signKP);
            Verify(GOST3410EL, _data, _signKP, _sigBytes);
        }
        log(runTime("Sign and Verify Test for GOST3410EL ", _start));

    }

    public void testSign() {

        String _data = "Test data";
        byte[] _sigBytes = new byte[signLen];

        KeyPair _signKP = null;
        long _start = 0;

        //_signKP = makeKeyPair(GOST3410);
        //_start = getTime();
        //for (int i = 0; i < testsCount; i++) {
        //    _sigBytes = Sign(GOST3410, _data, _signKP);
        //}
        //log(runTime("Sign Test for GOST3410 ", _start));

        _signKP = makeKeyPair(GOST3410EL);
        _start = getTime();
        for (int i = 0; i < testsCount; i++) {
            _sigBytes = Sign(GOST3410EL, _data, _signKP);
        }
        log(runTime("Sign Test for GOST3410EL ", _start));

    }

    public void testVerify() {

        String _data = "Test data";
        byte[] _sigBytes = new byte[signLen];

        KeyPair _signKP = null;
        long _start = 0;

        //_signKP = makeKeyPair(GOST3410);
        //_sigBytes = Sign(GOST3410, _data, _signKP);
        //
        //_start = getTime();
        //for (int i = 0; i < testsCount; i++) {
        //    Verify(GOST3410, _data, _signKP, _sigBytes);
        //}
        //log(runTime("Verify Test for GOST3410 ", _start));

        _signKP = makeKeyPair(GOST3410EL);
        _sigBytes = Sign(GOST3410EL, _data, _signKP);

        _start = getTime();
        for (int i = 0; i < testsCount; i++) {
            Verify(GOST3410EL, _data, _signKP, _sigBytes);
        }
        log(runTime("Verify Test for GOST3410EL ", _start));
    }

   public void testKeyPair() {

        try {
            long _start = 0;

            //_start = getTime();
            //for (int i = 0; i < testsCount; i++) {
            //    KeyPair _signKP = makeKeyPair(GOST3410);
            //}
            //log(runTime("KeyPair Test for GOST3410 ", _start));

            _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                KeyPair _signKP = makeKeyPair(GOST3410EL);
            }
            log(runTime("KeyPair Test for GOST3410EL ", _start));

            //_start = getTime();
            //for (int i = 0; i < testsCount; i++) {
            //    KeyPair _signKP = makeKeyPair(GOSTDH);
            //}
            //log(runTime("KeyPair Test for GOSTDH ", _start));

            _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                KeyPair _signKP = makeKeyPair(GOSTECDH);
            }
            log(runTime("KeyPair Test for GOSTECDH ", _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }

    public void testCertificate() {

        try {

            String _subDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
            String _issDN = "CN=CA, E=info@digt.ru, O=DIGT, C=RU";
            KeyPair _issKP = makeKeyPair(GOST3410EL);
            long _start = getTime();

            for (int i = 0; i < testsCount; i++) {
                KeyPair _subKP = makeKeyPair(GOST3410EL);
                X509Certificate _devCert = KeyPairAndCertificateUtil.makeCertificate(_subKP, _subDN, _issKP, _issDN, "GOST3411WithGOST3410EL");
            }
            log(runTime("Certificate Test for GOST3410 ", _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }
    }


    public void testMultiThread() {
        new TestThread().runTest();
    }

    /*
    *
    *  INTERNAL METHODS
    *
    */
    private void SignVerify(int _thread) {
        try {

            String _data = "Test data";
            byte[] _sigBytes = new byte[signLen];
            //KeyPair _signKP = makeKeyPair(GOST3410);
            KeyPair _signKPEL = makeKeyPair(GOST3410EL);
            long _start = getTime();
            for (int i = 0; i < testsCount; i++) {
                //_sigBytes = Sign(GOST3410, _data, _signKP);
                //Verify(GOST3410, _data, _signKP, _sigBytes);
                _sigBytes = Sign(GOST3410EL, _data, _signKPEL);
                Verify(GOST3410EL, _data, _signKPEL, _sigBytes);
            }
            log(runTime("SignVerify Stress Test for Thread Number " + _thread , _start));
        } catch (Exception ex) {
            log(ex);
            fail();
        }

    }


    private byte[] Sign(int _algo, String _data, KeyPair _signKP) {
        try {

            Signature sig = Signature.getInstance(getAlgo(_algo), "DIGT");
            PrivateKey privKey = _signKP.getPrivate();
            sig.initSign(privKey);
            sig.update(_data.getBytes());
            return sig.sign();

        } catch (Exception ex) {
            log(ex);
            fail();
        }
        return null;
    }

    private void Verify(int _algo, String _data, KeyPair _signKP, byte[] _sigBytes) {
        try {

            boolean status = false;
            Signature sig = Signature.getInstance(getAlgo(_algo), "DIGT");
            PublicKey pubKey = _signKP.getPublic();
            sig.initVerify(pubKey);
            sig.update(_data.getBytes());
            status = sig.verify(_sigBytes);
            log("Signature verification status = " + status);
            assertEquals(true, status);

        } catch (Exception ex) {
            log(ex);
            fail();
          }
    }

    private KeyPair makeKeyPair(int _algo) {
        switch (_algo) {
            case GOST3410:
                return KeyPairAndCertificateUtil.makeGOST3410KeyPair();
            case GOST3410EL:
                return KeyPairAndCertificateUtil.makeGOST3410ELKeyPair();
            case GOSTDH:
                return KeyPairAndCertificateUtil.makeGOSTDHKeyPair();
            case GOSTECDH:
                return KeyPairAndCertificateUtil.makeGOSTELDHKeyPair();
            default:
                return null;
        }
    }

    private String getAlgo(int _algo) {
        switch (_algo) {
            case GOST3410:
                return "GOST3410";
            case GOST3410EL:
                return "GOST3410EL";
            case GOSTDH:
                return "GOSTDH";
            case GOSTECDH:
                return "GOSTECDH";
            default:
                return "";
        }
    }


    private String runTime(String _testName, long _start) {
        long _runTime = getTime() - _start;
        return _testName + " was run " + testsCount + " times  in " + _runTime / 1000.00 + " second.";
    }

    private long getTime() {
        Calendar cal = Calendar.getInstance();
        return cal.getTimeInMillis();
    }


    private class TestThread extends Thread {

        public void run() {
            int _thread = ++threads;
            log("Thread Number " + _thread + " started");
            SignVerify(_thread);
            log("Thread Number " + _thread + " stopped");
            --threads;
        }

        public void runTest() {
            try {
                for (int i = 0; i < threadCount; i++) {
                    new TestThread().start();
                }
                sleep(1000);
                while(threads>0) {
                    sleep(100);
                }
                log("Test finished");
            } catch (Exception ex) {
                log(ex);
                fail();
            }
        }

    }
}
