/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.jce.test;

import com.digt.trusted.asn1.ASN1InputStream;
import com.digt.trusted.asn1.ASN1Sequence;
import com.digt.trusted.asn1.DERSequence;
import com.digt.trusted.asn1.x509.*;
import com.digt.trusted.crypto.params.GOST3410KeyGenParameterSpec;
import com.digt.trusted.jce.PKCS10CertificationRequest;
import com.digt.trusted.jce.X509KeyUsage;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.x509.X509V3CertificateGenerator;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.*;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Random;

/**
 * User: andy
 * Date: 01.06.2005
 * Time: 15:15:15
 */
public class KeyPairAndCertificateUtil {

    public static KeyPairGenerator gost3410kpg;
    public static KeyPairGenerator gost3410ELkpg;
    public static KeyPairGenerator gost3410v12256kpg;
    public static KeyPairGenerator gost3410v12512kpg;
    public static KeyPairGenerator gostDHkpg;
    public static KeyPairGenerator gostELDHkpg;
    public static KeyPairGenerator gostv12256DHkpg;
    public static KeyPairGenerator gostv12512DHkpg;
    //  
    public static BigInteger serialNumber;
    //     
    public static String pin;
    //  
    public static String container;

    public static final boolean DEBUG = true;

    private static void _init() {
        try {
            Security.addProvider(new DIGTProvider());

            pin = "11111111";

            //   "GOST3410"   34.11/34.10-94,     
            //container = allocateContainer() + "GOST3410";
            //log("container = " + container);
            //gost3410kpg = KeyPairGenerator.getInstance("GOST3410", "DIGT");
            //gost3410kpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOST3410EL"   34.11/34.10-2001,     
            container = allocateContainer() + "GOST3410EL";
            log("container = " + container);
            gost3410ELkpg = KeyPairGenerator.getInstance("GOST3410EL", "DIGT");
            gost3410ELkpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOST3410v12256"   34.11-12/34.10-12 (256 bit),     
            container = allocateContainer() + "GOST3410v12256";
            log("container = " + container);
            gost3410v12256kpg = KeyPairGenerator.getInstance("GOST3410v12256", "DIGT");
            gost3410v12256kpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOST3410v12512"   34.11-12/34.10-12 (512 bit),     
            container = allocateContainer() + "GOST3410v12512";
            log("container = " + container);
            gost3410v12512kpg = KeyPairGenerator.getInstance("GOST3410v12512", "DIGT");
            gost3410v12512kpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOSTDH"   34.11/34.10-94   AT_KEYEXCHANGE  
            //  Diffie-Hellman        
            //container = allocateContainer() + "GOSTDH";
            //log("container = " + container);
            //gostDHkpg = KeyPairGenerator.getInstance("GOSTDH", "DIGT");
            //gostDHkpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOSTECDH"   34.11/34.10-2001   AT_KEYEXCHANGE  
            //  Diffie-Hellman        
            container = allocateContainer() + "GOSTECDH";
            log("container = " + container);
            gostELDHkpg = KeyPairGenerator.getInstance("GOSTECDH", "DIGT");
            gostELDHkpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOSTv12256DH"   34.11-12/34.10-12   AT_KEYEXCHANGE  
            //  Diffie-Hellman        
            container = allocateContainer() + "GOSTv12256DH";
            log("container = " + container);
            gostv12256DHkpg = KeyPairGenerator.getInstance("GOSTv12256DH", "DIGT");
            gostv12256DHkpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            //   "GOSTv12512DH"   34.11-12/34.10-12   AT_KEYEXCHANGE  
            //  Diffie-Hellman        
            container = allocateContainer() + "GOSTv12512DH";
            log("container = " + container);
            gostv12512DHkpg = KeyPairGenerator.getInstance("GOSTv12512DH", "DIGT");
            gostv12512DHkpg.initialize(new GOST3410KeyGenParameterSpec(container, pin));

            serialNumber = new BigInteger(64, new Random());

        } catch (Exception ex) {
            log(ex);
        }
    }

    //  
    public static X509Certificate makeCertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN, String SigAlg, boolean _ca, boolean _tsp)
            throws GeneralSecurityException, IOException {

        //     
        PublicKey _subPub = _subKP.getPublic();
        PrivateKey _issPriv = _issKP.getPrivate();
        PublicKey _issPub = _issKP.getPublic();

        X509V3CertificateGenerator _v3CertGen = new X509V3CertificateGenerator();

        //  
        _v3CertGen.reset();
        _v3CertGen.setSerialNumber(allocateSerialNumber());
        _v3CertGen.setIssuerDN(new X509Name(_issDN));
        _v3CertGen.setNotBefore(new Date(System.currentTimeMillis()));
        _v3CertGen.setNotAfter(new Date(System.currentTimeMillis() + (1000L * 60 * 60 * 24 * 100)));
        _v3CertGen.setSubjectDN(new X509Name(_subDN));
        _v3CertGen.setPublicKey(_subPub);
        _v3CertGen.setSignatureAlgorithm(SigAlg);

        _v3CertGen.addExtension(X509Extensions.SubjectKeyIdentifier,
                false,
                createSubjectKeyId(_subPub));

        _v3CertGen.addExtension(X509Extensions.AuthorityKeyIdentifier,
                false,
                createAuthorityKeyId(_issPub));

        if (_ca)
        {
            _v3CertGen.addExtension(X509Extensions.BasicConstraints,
                false,
                new BasicConstraints(_ca));
        }


        if (_tsp)
        {
            // extended key usage timestamping
            _v3CertGen.addExtension(X509Extensions.ExtendedKeyUsage, true,
                    new ExtendedKeyUsage(new DERSequence(
                            KeyPurposeId.id_kp_timeStamping)));
        }
        else
        {
            //key usage - sign, encrypt.
            _v3CertGen.addExtension("2.5.29.15", false,
                    new X509KeyUsage(X509KeyUsage.digitalSignature | X509KeyUsage.dataEncipherment));
        }
        // 
        X509Certificate _cert = _v3CertGen.generateX509Certificate(_issPriv);

        //       
        _cert.checkValidity(new Date());
        _cert.verify(_issPub);

        //        .
        installCertificate(_subKP.getPrivate(), _cert);

        return _cert;
    }

    //   
    public static void installCertificate(PrivateKey _subPriv, X509Certificate _cert)
            throws NoSuchProviderException, KeyStoreException, IOException, NoSuchAlgorithmException, CertificateException {
        //      CSP  
        //   .
        KeyStore store = KeyStore.getInstance("CryptoProCSPKeyStore", "DIGT");
        store.load(new ByteArrayInputStream("CurrentUser".getBytes("UTF-8")), null);
        X509Certificate[] chain = {_cert};
        store.setKeyEntry("private", _subPriv, pin.toCharArray(), chain);
    }

    //      PKCS#10
    public static byte[] makeCertificateRequest(KeyPair _subKP, String _subDN, String SigAlg)
            throws NoSuchProviderException, NoSuchAlgorithmException, InvalidKeyException, SignatureException {

        X509Name reqDN = new X509Name(_subDN);
        // create request
        PKCS10CertificationRequest reqch1 = new PKCS10CertificationRequest(SigAlg,
                reqDN,
                _subKP.getPublic(),
                null,
                _subKP.getPrivate());

        return reqch1.getEncoded();
    }

    public synchronized static KeyPair makeGOST3410v12512KeyPair() {
        _init();
        return gost3410v12512kpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOST3410v12256KeyPair() {
        _init();
        return gost3410v12256kpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOST3410ELKeyPair() {
        _init();
        return gost3410ELkpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOST3410KeyPair() {
        _init();
        return gost3410kpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOSTv12512DHKeyPair() {
        _init();
        return gostv12512DHkpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOSTv12256DHKeyPair() {
        _init();
        return gostv12256DHkpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOSTELDHKeyPair() {
        _init();
        return gostELDHkpg.generateKeyPair();
    }

    public synchronized static KeyPair makeGOSTDHKeyPair() {
        _init();
        return gostDHkpg.generateKeyPair();
    }

    public static X509Certificate makeCertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN, String SigAlg)
            throws GeneralSecurityException, IOException {

        return makeCertificate(_subKP, _subDN, _issKP, _issDN, SigAlg, false, false);
    }

    public static X509Certificate makeTSPCertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN, String SigAlg)
            throws GeneralSecurityException, IOException {

        return makeCertificate(_subKP, _subDN, _issKP, _issDN, SigAlg, false, true);
    }

    public static X509Certificate makeCACertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN, String SigAlg)
            throws GeneralSecurityException, IOException {

        return makeCertificate(_subKP, _subDN, _issKP, _issDN, SigAlg, true, false);
    }

    public static String getPin() {
        return pin;
    }

    public static String allocateContainer() {
        String _tmp = "TestAt" + System.currentTimeMillis()+ Thread.currentThread().getName();
        if (System.getProperty("os.name").toLowerCase().indexOf("windows") < 0) {
            // Key container is specified. Under UNIX it has
            // "\\\\.\\hdimage\\<Container name>" format.
            // Please refer to CSP docs for details
            _tmp = "\\\\.\\hdimage\\" + _tmp;
        }
        return _tmp;
    }

    /*
    *
    *  INTERNAL METHODS
    *
    */

    private static AuthorityKeyIdentifier createAuthorityKeyId(PublicKey _pubKey)
            throws IOException, NoSuchAlgorithmException
    {

        ByteArrayInputStream _bais = new ByteArrayInputStream(_pubKey.getEncoded());
        SubjectPublicKeyInfo _info = new SubjectPublicKeyInfo((ASN1Sequence) new ASN1InputStream(_bais).readObject());

        return new AuthorityKeyIdentifier(_info);
    }

    private static AuthorityKeyIdentifier createAuthorityKeyId(PublicKey _pubKey, X509Name _name, int _sNumber)
            throws IOException, NoSuchAlgorithmException
    {

        ByteArrayInputStream _bais = new ByteArrayInputStream(_pubKey.getEncoded());
        SubjectPublicKeyInfo _info = new SubjectPublicKeyInfo((ASN1Sequence) new ASN1InputStream(_bais).readObject());

        GeneralName _genName = new GeneralName(_name);

        return new AuthorityKeyIdentifier(_info, new GeneralNames(new DERSequence(_genName)), BigInteger.valueOf(_sNumber));

    }

    private static SubjectKeyIdentifier createSubjectKeyId(PublicKey _pubKey)
            throws IOException, NoSuchAlgorithmException 
    {

        ByteArrayInputStream _bais = new ByteArrayInputStream(_pubKey.getEncoded());
        SubjectPublicKeyInfo _info = new SubjectPublicKeyInfo((ASN1Sequence) new ASN1InputStream(_bais).readObject());
        return new SubjectKeyIdentifier(_info);
    }

    private static BigInteger allocateSerialNumber() {
        BigInteger _tmp = serialNumber;
        serialNumber = serialNumber.add(BigInteger.ONE);
        return _tmp;
    }

    public static void log(Exception _ex) {
        if (DEBUG) {
            _ex.printStackTrace();
        }
    }

    public static void log(String _msg) {
        if (DEBUG) {
            System.out.println(_msg);
        }
    }
}
