/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.ocsp;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.math.BigInteger;
import java.security.KeyPair;
import java.security.Security;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.security.cert.CertificateFactory;
import java.util.Hashtable;

import com.digt.trusted.asn1.DEROctetString;
import com.digt.trusted.asn1.ocsp.OCSPObjectIdentifiers;
import com.digt.trusted.asn1.x509.GeneralName;
import com.digt.trusted.asn1.x509.X509Extension;
import com.digt.trusted.asn1.x509.X509Extensions;
import com.digt.trusted.asn1.x509.X509Name;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.provider.JDKGOST3410ELPrivateKey;
import com.digt.trusted.jce.provider.JDKGOST3410ELPublicKey;
import com.digt.trusted.ocsp.BasicOCSPResp;
import com.digt.trusted.ocsp.CertificateID;
import com.digt.trusted.ocsp.OCSPReq;
import com.digt.trusted.ocsp.OCSPReqGenerator;
import com.digt.trusted.ocsp.OCSPResp;
import com.digt.trusted.ocsp.Req;
import com.digt.trusted.ocsp.RespData;
import com.digt.trusted.ocsp.SingleResp;
import com.digt.trusted.util.encoders.Base64;
import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class OCSPTest
    extends TestCase
{
    byte[]  testResp1 = Base64.decode(
                "MIIE+woBAKCCBPQwggTwBgkrBgEFBQcwAQEEggThMIIE3TCByaEzMDExEzAR"
              + "BgkqhkiG9w0BCQEWBG9jc3AxCzAJBgNVBAYTAlJVMQ0wCwYDVQQDEwRvY3Nw"
              + "GBYyMDA0MTIwOTEyMzcyMC4wNTMwMTdaMHoweDBcMAoGBiqFAwICCQUABCBM"
              + "nlM7NWBXDm32IhPzSdPqWgrJK4jNuUgVicW4iGgMBgQgsGtZELqECoG0oYYH"
              + "HbXjroqayoZgnRqEWEpJt6xsabgCChEIJD0AAAAAAJSAABgWMjAwNDEyMDkx"
              + "MjM3MjAuMDM1MzE4WjAIBgYqhQMCAgMDQQADrICr+bavww1SOuMaivj2Ojp+"
              + "MF/j0WHwNPLFpiBRoslcMa8jXj4YM5ovHh2cA/46rB1tGvxdRRUaV/Whej8i"
              + "oIIDwDCCA7wwggO4MIIDZaADAgECAgoRGLbLAAAAAACSMAoGBiqFAwICAwUA"
              + "MH4xGjAYBgkqhkiG9w0BCQEWC3NiYUBkaWd0LnJ1MQswCQYDVQQGEwJSVTEM"
              + "MAoGA1UECBMDTUVMMRQwEgYDVQQHEwtZb3Noa2FyLU9sYTENMAsGA1UEChME"
              + "RGlndDEPMA0GA1UECxMGQ3J5cHRvMQ8wDQYDVQQDEwZzYmEtQ0EwHhcNMDQx"
              + "MjA2MTIxNzM4WhcNMDUxMjA2MTIyNzM4WjAxMRMwEQYJKoZIhvcNAQkBFgRv"
              + "Y3NwMQswCQYDVQQGEwJSVTENMAsGA1UEAxMEb2NzcDBjMBwGBiqFAwICEzAS"
              + "BgcqhQMCAiQABgcqhQMCAh4BA0MABED6oMTBIDU59nR1lLA0JOauN6xrncy4"
              + "xNz4zespr7ULgJ0ZyjedyKGsK5E2ajDTaFVCvEjLi9oVEc94sEOa3cfPo4IC"
              + "DDCCAggwEwYDVR0lBAwwCgYIKwYBBQUHAwkwHQYDVR0OBBYEFJ5Ptg2Y3sZs"
              + "I9OzY29HINU4IsmyMIG5BgNVHSMEgbEwga6AFEkmXctw0zGRkjzXxJN+iLee"
              + "DXGgoYGDpIGAMH4xGjAYBgkqhkiG9w0BCQEWC3NiYUBkaWd0LnJ1MQswCQYD"
              + "VQQGEwJSVTEMMAoGA1UECBMDTUVMMRQwEgYDVQQHEwtZb3Noa2FyLU9sYTEN"
              + "MAsGA1UEChMERGlndDEPMA0GA1UECxMGQ3J5cHRvMQ8wDQYDVQQDEwZzYmEt"
              + "Q0GCEHvWQroJuNqUSjXBw/OOus0wcQYDVR0fBGowaDAxoC+gLYYraHR0cDov"
              + "L3NiYS5kaWd0LmxvY2FsL0NlcnRFbnJvbGwvc2JhLUNBLmNybDAzoDGgL4Yt"
              + "ZmlsZTovL1xcc2JhLmRpZ3QubG9jYWxcQ2VydEVucm9sbFxzYmEtQ0EuY3Js"
              + "MIGiBggrBgEFBQcBAQSBlTCBkjBGBggrBgEFBQcwAoY6aHR0cDovL3NiYS5k"
              + "aWd0LmxvY2FsL0NlcnRFbnJvbGwvc2JhLmRpZ3QubG9jYWxfc2JhLUNBLmNy"
              + "dDBIBggrBgEFBQcwAoY8ZmlsZTovL1xcc2JhLmRpZ3QubG9jYWxcQ2VydEVu"
              + "cm9sbFxzYmEuZGlndC5sb2NhbF9zYmEtQ0EuY3J0MAoGBiqFAwICAwUAA0EA"
              + "BhmqdDGpXvuegi3+cIIcC5sdbXiiM5JsxJBzVHhthvH5oMRjgCPGFtdhQZ2M"
              + "trD+8b5kye4SkM3bBf+vlcOwsQ==");

    public String getName()
    {
        return "OCSP";
    }

   public void testOCSPGOST()
    {
        try
        {
           String          signDN = "O=DIGT, C=RU";
           KeyPair         signKP = OCSPTestUtil.makeGOSTKeyPair();
           X509Certificate testCert = OCSPTestUtil.makeGOSTCertificate(signKP, signDN, signKP, signDN);

           String          origDN = "CN=Sergey A. Bahtin, O=DIGT, C=RU";
            GeneralName     origName = new GeneralName(new X509Name(origDN));

            //
            // general id value for our test issuer cert and a serial number.
            //
           CertificateID   id = new CertificateID(CertificateID.HASH_GOST3411, testCert, BigInteger.valueOf(1));

            //
            // basic request generation
            //
            OCSPReqGenerator    gen = new OCSPReqGenerator();

            gen.addRequest(
                   new CertificateID(CertificateID.HASH_GOST3411, testCert, BigInteger.valueOf(1)));

            OCSPReq         req = gen.generate();

            assertTrue(req.isSigned());

            Req[]           requests = req.getRequestList();

            assertTrue(requests[0].getCertID().equals(id));

            //
            // request generation with signing
            //
            X509Certificate[]   chain = new X509Certificate[1];

            gen = new OCSPReqGenerator();

            gen.addRequest(
                   new CertificateID(CertificateID.HASH_GOST3411, testCert, BigInteger.valueOf(1)));

            chain[0] = testCert;

           req = gen.generate("GOST3411withGOST3410EL", signKP.getPrivate(), chain, "DIGT");

            assertTrue(req.isSigned());

            assertTrue(req.verify(signKP.getPublic(), "DIGT"));

            requests = req.getRequestList();

            assertTrue(requests[0].getCertID().equals(id));

            //
           // response parsing - test 1
            //
           OCSPResp    response = new OCSPResp(new ByteArrayInputStream(testResp1));

           assertFalse(response.getStatus() != 0);

           BasicOCSPResp       brep = (BasicOCSPResp)response.getResponseObject();
           chain = brep.getCerts("DIGT");
            
           assertTrue(brep.verify(chain[0].getPublicKey(), "DIGT"));

           //
           // test 2
           //
           RespData            respData = brep.getResponseData();
           SingleResp[]        singleResp = respData.getResponses();

           response = new OCSPResp(new ByteArrayInputStream(testResp1));

           assertFalse(response.getStatus() != 0);

           brep = (BasicOCSPResp)response.getResponseObject();
           chain = brep.getCerts("DIGT");

           assertTrue(brep.verify(chain[0].getPublicKey(), "DIGT"));

           respData = brep.getResponseData();
           singleResp = respData.getResponses();
       }
       catch (Exception e)
       {
            fail(e.getMessage());
       }
   }

   //before test need: created KeyStore = test.gks, with alias "TestOCSP" stored private key and certificate
   public void TestOCSPGOST2()
   {
       try
       {
           char[]   passwd = { 'h', 'e', 'l', 'l', 'o', ' ', 'w', 'o', 'r', 'l', 'd' };
           X509Certificate testCert = null;
           X509Certificate testCertIssuer = null;

           JDKGOST3410ELPrivateKey sKey = null;
           JDKGOST3410ELPublicKey vKey = null;

           KeyStore store = KeyStore.getInstance("GKS", "DIGT");

           String pathto = System.getProperty("user.home") + System.getProperty("file.separator");

           //-----get CA certificate from file-----//
           CertificateFactory cf = CertificateFactory.getInstance("X.509", "DIGT");
           DataInputStream in = new DataInputStream( new BufferedInputStream(
                                                       new FileInputStream(System.getProperty("user.home")+System.getProperty("file.separator")+"sbaCA.cer") ));
           byte[] certbytes = new byte[in.available()];
           in.read(certbytes);
           in.close();
           testCertIssuer = (X509Certificate)cf.generateCertificate(new ByteArrayInputStream(certbytes));
           //--------------------------------------------//

           //get created store
           FileInputStream fis = new FileInputStream(pathto+"test.gks");
           store.load(fis, passwd);

           if (store.containsAlias("TestOCSP"))
           {
              testCert = (X509Certificate)store.getCertificate("TestOCSP"); //get certificate by keystore alias

              String alias = store.getCertificateAlias(testCert); //get keystore alias by cert

              sKey = (JDKGOST3410ELPrivateKey)store.getKey(alias, passwd);
              vKey = (JDKGOST3410ELPublicKey)testCert.getPublicKey();
           }

           //
           // general id value for our test issuer cert and a serial number.
           //

           CertificateID   id = new CertificateID(CertificateID.HASH_GOST3411, testCertIssuer, testCert.getSerialNumber());

           //
           // basic request generation
           //
           OCSPReqGenerator    gen = new OCSPReqGenerator();

           gen.addRequest(id);

           OCSPReq         req = gen.generate();

           assertFalse(req.isSigned());

           Req[]           requests = req.getRequestList();

           assertTrue(requests[0].getCertID().equals(id));

           //
           // request generation with signing
           //
           X509Certificate[]   chain = new X509Certificate[1];

           gen = new OCSPReqGenerator();

           gen.addRequest(id);

            chain[0] = testCert;

           req = gen.generate("GOST3411withGOST3410EL", sKey, chain, "DIGT");

            assertTrue(req.isSigned());

           assertTrue(req.verify(vKey, "DIGT"));

            requests = req.getRequestList();

            assertTrue (requests[0].getCertID().equals(id));

//out request to file
/*DataOutputStream out1 = new DataOutputStream( new BufferedOutputStream(new FileOutputStream("c:\\bouncyOCSPGOST.req")));
for (int i = 0; i != req.getEncoded().length; i++) out1.writeByte(req.getEncoded()[i]);
out1.close();*/

            //
            // response parsing - test 1
            //
            OCSPResp    response = new OCSPResp(new ByteArrayInputStream(testResp1));

            assertFalse(response.getStatus() != 0);

            BasicOCSPResp       brep = (BasicOCSPResp)response.getResponseObject();
           chain = brep.getCerts("DIGT");

           assertTrue(brep.verify(chain[0].getPublicKey(), "DIGT"));

            //
            // test 2
            //
            RespData            respData = brep.getResponseData();
            SingleResp[]        singleResp = respData.getResponses();

           response = new OCSPResp(new ByteArrayInputStream(testResp1));

            assertFalse(response.getStatus() != 0);

            brep = (BasicOCSPResp)response.getResponseObject();
           chain = brep.getCerts("DIGT");

           assertTrue(!brep.verify(chain[0].getPublicKey(), "DIGT"));

            respData = brep.getResponseData();
            singleResp = respData.getResponses();
        }
        catch (Exception e)
        {
            fail(e.getMessage());
        }

    }


    public static void main(String args[]) {
        junit.textui.TestRunner.run(OCSPTest.class);
    }

    public static Test suite() {
        return new TestSuite(OCSPTest.class);
    }


}
