/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.ocsp;

import java.io.*;
import java.math.BigInteger;
import java.security.*;
import java.security.cert.X509Certificate;
import java.util.Date;

import javax.crypto.KeyGenerator;

import com.digt.trusted.asn1.ASN1InputStream;
import com.digt.trusted.asn1.ASN1Sequence;
import com.digt.trusted.asn1.DERSequence;
import com.digt.trusted.asn1.x509.AuthorityKeyIdentifier;
import com.digt.trusted.asn1.x509.BasicConstraints;
import com.digt.trusted.asn1.x509.GeneralName;
import com.digt.trusted.asn1.x509.GeneralNames;
import com.digt.trusted.asn1.x509.SubjectKeyIdentifier;
import com.digt.trusted.asn1.x509.SubjectPublicKeyInfo;
import com.digt.trusted.asn1.x509.X509Extensions;
import com.digt.trusted.asn1.x509.X509Name;
import com.digt.trusted.x509.X509V3CertificateGenerator;
import com.digt.trusted.jce.X509KeyUsage;
import com.digt.trusted.crypto.params.GOST3410KeyGenParameterSpec;

public class OCSPTestUtil
{
    
    public static SecureRandom     rand;
    public static KeyPairGenerator gostkpg;
    public static BigInteger       serialNumber;
    
    public static final boolean DEBUG = true;
    
    static {
        try {
            rand = new SecureRandom();

            gostkpg  = KeyPairGenerator.getInstance("GOST3410EL", "DIGT");
            gostkpg.initialize(new GOST3410KeyGenParameterSpec("TestOCSP"));

            serialNumber = new BigInteger("1");
        }
        catch(Exception ex) {
            log(ex);
        }
    }
    
   public static KeyPair makeGOSTKeyPair() {
       return gostkpg.generateKeyPair();
    }
    
   public static X509Certificate makeGOSTCertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN)
        throws GeneralSecurityException, IOException {
        
       return makeGOSTCertificate(_subKP, _subDN, _issKP, _issDN, false);
    }
    
    public static X509Certificate makeCACertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN) 
        throws GeneralSecurityException, IOException {
        
		return makeGOSTCertificate(_subKP, _subDN, _issKP, _issDN, true);
    }
    
   public static X509Certificate makeGOSTCertificate(KeyPair _subKP, String _subDN, KeyPair _issKP, String _issDN, boolean _ca)
        throws GeneralSecurityException, IOException {

        PublicKey  _subPub  = _subKP.getPublic();
        PrivateKey _issPriv = _issKP.getPrivate();
        PublicKey  _issPub  = _issKP.getPublic();
        
        X509V3CertificateGenerator _v3CertGen = new X509V3CertificateGenerator();
        
        _v3CertGen.reset();
        _v3CertGen.setSerialNumber(allocateSerialNumber());
        _v3CertGen.setIssuerDN(new X509Name(_issDN));
        _v3CertGen.setNotBefore(new Date(System.currentTimeMillis()));
        _v3CertGen.setNotAfter(new Date(System.currentTimeMillis() + (1000L * 60 * 60 * 24 * 100)));
        _v3CertGen.setSubjectDN(new X509Name(_subDN));
        _v3CertGen.setPublicKey(_subPub);
       _v3CertGen.setSignatureAlgorithm("GOST3411WithGOST3410EL");

        _v3CertGen.addExtension(
            X509Extensions.SubjectKeyIdentifier,
            false,
            createSubjectKeyId(_subPub));

        _v3CertGen.addExtension(
            X509Extensions.AuthorityKeyIdentifier,
            false,
            createAuthorityKeyId(_issPub));

        _v3CertGen.addExtension(
            X509Extensions.BasicConstraints,
            false,
            new BasicConstraints(_ca));

       //key usage - sign, encrypt.
       _v3CertGen.addExtension("2.5.29.15", false,
               new X509KeyUsage(X509KeyUsage.digitalSignature | X509KeyUsage.dataEncipherment));

        X509Certificate _cert = _v3CertGen.generateX509Certificate(_issPriv);

        _cert.checkValidity(new Date());
        _cert.verify(_issPub);

///out cert
       DataOutputStream out = new DataOutputStream( new BufferedOutputStream(new FileOutputStream("c:\\ocspTest.cer")));
       for (int i = 0; i != _cert.getEncoded().length; i++) out.writeByte(_cert.getEncoded()[i]);
       out.close();
//end out cert

        return _cert;
    }
    

    /*  
     *  
     *  INTERNAL METHODS
     *  
     */ 
    
    private static AuthorityKeyIdentifier createAuthorityKeyId(PublicKey _pubKey) 
        throws IOException, NoSuchAlgorithmException
    {
        
        ByteArrayInputStream _bais = new ByteArrayInputStream(_pubKey.getEncoded());
        SubjectPublicKeyInfo _info = new SubjectPublicKeyInfo(
            (ASN1Sequence)new ASN1InputStream(_bais).readObject());

        return new AuthorityKeyIdentifier(_info);
    }

    private static AuthorityKeyIdentifier createAuthorityKeyId(PublicKey _pubKey, X509Name _name, int _sNumber) 
        throws IOException, NoSuchAlgorithmException
    {
        
        ByteArrayInputStream _bais = new ByteArrayInputStream(_pubKey.getEncoded());
        SubjectPublicKeyInfo _info = new SubjectPublicKeyInfo(
            (ASN1Sequence)new ASN1InputStream(_bais).readObject());

        GeneralName             _genName = new GeneralName(_name);

        return new AuthorityKeyIdentifier(
            _info, new GeneralNames(new DERSequence(_genName)), BigInteger.valueOf(_sNumber));

    }
    
    private static SubjectKeyIdentifier createSubjectKeyId(PublicKey _pubKey) 
        throws IOException, NoSuchAlgorithmException
    {
        
        ByteArrayInputStream _bais = new ByteArrayInputStream(_pubKey.getEncoded());
        SubjectPublicKeyInfo _info = new SubjectPublicKeyInfo(
            (ASN1Sequence)new ASN1InputStream(_bais).readObject());
        return new SubjectKeyIdentifier(_info);
    }

    private static BigInteger allocateSerialNumber() {
        BigInteger _tmp = serialNumber;
        serialNumber = serialNumber.add(BigInteger.ONE);
        return _tmp;
    }
    
    
    public static void log(Exception _ex) {
        if(DEBUG) {
            _ex.printStackTrace();
        }
    }

    public static void log(String _msg) {
        if(DEBUG) {
            System.out.println(_msg);
        }
    }
}
