/*
* Copyright(C) 1999-2008  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.tsp.test;

import com.digt.trusted.asn1.cms.AttributeTable;
import com.digt.trusted.asn1.pkcs.PKCSObjectIdentifiers;
import com.digt.trusted.cms.CMSProcessableByteArray;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.test.KeyPairAndCertificateUtil;
import com.digt.trusted.tsp.TSPHelper;
import com.digt.trusted.tsp.TimeStampToken;
import junit.framework.TestCase;
import junit.textui.TestRunner;

import java.security.KeyPair;
import java.security.Security;
import java.security.cert.X509Certificate;

/**
 *      /  
 */
public class TSPTest extends TestCase {

    //private KeyPair origKP94;
    private KeyPair origKP2001;

    private static final int GOST3410 = 1;
    private static final int GOST3410EL = 2;
    private static final int GOSTDH = 3;
    private static final int GOSTECDH = 4;

    /**
     *      
     */
    private static final String tsaUrl = "http://www.cryptopro.ru/tsp/tsp.srf";

    private X509Certificate signCert;
    //private X509Certificate origCert94;
    private X509Certificate origCert2001;

    public String getName() {
        return "TSP Test";
    }

    protected void setUp()
    {
        Security.addProvider(new DIGTProvider());
    }

    private void init() throws Exception
    {
        String origDN = "CN=Developer, E=info@digt.ru, O=DIGT, C=RU";
        String signDN = "CN=CA, E=info@digt.ru, O=DIGT, C=RU";

        KeyPair signKP = makeKeyPair(GOST3410EL);
        signCert = KeyPairAndCertificateUtil.makeCACertificate(
                signKP, signDN, signKP, signDN, "GOST3411WithGOST3410EL");

        //origKP94 = makeKeyPair(GOST3410);
        //origCert94 = KeyPairAndCertificateUtil.makeTSPCertificate(
        //        origKP94, origDN, signKP, signDN, "GOST3411WithGOST3410");

        origKP2001 = makeKeyPair(GOST3410EL);
        origCert2001 = KeyPairAndCertificateUtil.makeTSPCertificate(
                origKP2001, origDN, signKP, signDN, "GOST3411WithGOST3410EL");
    }

    /**
     *  /       
     * @throws Exception
     */
    public void testLocal()
        throws Exception
    {
        init();
        String testData = "Test data";
        TSPHelper tsp = new TSPHelper(new CMSProcessableByteArray(testData.getBytes()));

        TimeStampToken tsToken = tsp.getTimeStampLocal(origKP2001.getPrivate(), signCert, origCert2001);
        TSPHelper.validateCert(tsToken);

        AttributeTable  table = tsToken.getSignedAttributes();

        assertNotNull("no signingCertificate attribute found", table.get(PKCSObjectIdentifiers.id_aa_signingCertificate));
    }

    /**
     *  /       
     * @throws Exception
     */
    public void testOnline() throws Exception
    {
        String testData = "Test data";
        TSPHelper tsp = new TSPHelper(new CMSProcessableByteArray(testData.getBytes()));
        TimeStampToken  tsToken = tsp.getTimeStampOnline(tsaUrl);
        assertNotNull("No timestamp token returned", tsToken);

        TSPHelper.validateCert(tsToken);
    }

    private KeyPair makeKeyPair(int _algo) {
        switch (_algo) {
            case GOST3410:
                return KeyPairAndCertificateUtil.makeGOST3410KeyPair();
            case GOST3410EL:
                return KeyPairAndCertificateUtil.makeGOST3410ELKeyPair();
            case GOSTDH:
                return KeyPairAndCertificateUtil.makeGOSTDHKeyPair();
            case GOSTECDH:
                return KeyPairAndCertificateUtil.makeGOSTELDHKeyPair();
            default:
                return null;
        }
    }

    public static void main(String[] args)
    {
        TestRunner.run(TSPTest.class);
    }

}
