/*
 * Copyright(C) 1999-2018  << >>
 *
 *    , 
 *    << >>.
 *
 *        ,
 * ,    ,
 *     ,
 * ,     
 *     
 *     << >>.
 */

package com.digt.trusted.xmlsig.test;

import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.transforms.params.XPath2FilterContainer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.Security;
import java.security.cert.X509Certificate;

import java.util.Date;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import com.digt.trusted.crypto.params.GOST3410KeyGenParameterSpec;
import com.digt.trusted.jce.X509Principal;
import com.digt.trusted.jce.interfaces.GOST3410PrivateKey;
import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.x509.X509V3CertificateGenerator;
import com.digt.trusted.xmlsig.Consts;

/**
 *      
 *  XML-  -.
 *
 * @author algv
 */
public class Xml2SigDocCreate12256 {
    private static final String digUri = Consts.URN_GOST_DIGEST_2012_256;

    private static final String algUri = Consts.URN_GOST_SIGN_2012_256;

    private static final String keyalias1 = "GOST3410v12256-XML2SigDoc1";
    private static final String keyalias2 = "GOST3410v12256-XML2SigDoc2";

    /**
     * 
     * @param args
     */
    public static void main(String[] args) {
        // xml 
        String xmlfile = "data/Xml2Sig.xml";

        // xml 
        String xmlsigfile = "XmlTwoDSig12256.xml";

        String KeyPairAlgorithm = "GOST3410v12256";
        String SignAlgorithmName = "GOST3411v12256withGOST3410v12256";

        try {
            Security.addProvider(new DIGTProvider());

            //   
            System.out.print("Generating keypair1 ... ");
            KeyPair pair1 = GeneratePair(KeyPairAlgorithm, SignAlgorithmName, keyalias1);
            if (pair1 != null) {
                System.out.println("Done.");
            } else {
                System.out.println("KeyPair1 isn`t generated");
                return;
            }
            System.out.print("Generating keypair2 ... ");
            KeyPair pair2 = GeneratePair(KeyPairAlgorithm, SignAlgorithmName, keyalias2);
            if (pair2 != null) {
                System.out.println("Done.");
            } else {
                System.out.println("KeyPair2 isn`t generated");
                return;
            }

            //  
            System.out.print("Generating certificate1 ... ");
            X509Certificate cert1 = generateCert(pair1, SignAlgorithmName);
            if (cert1 != null) {
                System.out.println("Done.");
            } else {
                System.out.println("Certificate1 isn`t generated");
                return;
            }
            System.out.print("Generating certificate2 ... ");
            X509Certificate cert2 = generateCert(pair2, SignAlgorithmName);
            if (cert2 != null) {
                System.out.println("Done.");
            } else {
                System.out.println("Certificate2 isn`t generated");
                return;
            }

            //        
            if (args.length > 0) {
                String al = args[0];
                if ((new File(al)).exists()) {
                    xmlfile = al;
                }
            }

            /*
             *      
             * 
             *
             *    MSXML 5.0,  JCP (   Straight)
            init(Consts.CONFIG);
             *   (   Straight)
            init("resource/tj.xml");
             */
            init(Consts.CONFIG);

            System.out.println("Start signing file ...");
            //    
            String Id1 = "FirstSignature";

            //    
            String Id2 = "SecondSignature";

            twoSignDoc(pair1, pair2, cert1, cert2, Id1, Id2, algUri,
                    digUri, xmlfile, xmlsigfile);
            System.out.println("Created file: " + xmlsigfile);

            //  
            KeyStore ks = OpenKeyStore("CurrentUser/Containers");
            DeleteKey(ks, pair1);
            DeleteKey(ks, pair2);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     *    XML-   
     *
     * @param keyPair1   1
     * @param keyPair2   2
     * @param cert1   1
     * @param cert2   2
     * @param Id1   1
     * @param Id2   2
     * @param signMethod  
     * @param digestMethod  ,   
     * @param testDoc  
     * @param signDoc  
     * @throws Exception e
     */
    static void twoSignDoc(
            KeyPair keyPair1,
            KeyPair keyPair2,
            X509Certificate cert1,
            X509Certificate cert2,
            String Id1, String Id2,
            String signMethod, String digestMethod,
            String testDoc, String signDoc)
            throws Exception {

        //      
        PrivateKey privateKey1 = keyPair1.getPrivate();

        //      
        PrivateKey privateKey2 = keyPair2.getPrivate();

        /*   XML-   */

        //    XML-
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

        //  ,         XML-
        dbf.setIgnoringElementContentWhitespace(true);

        //  ,    CDATA      XML-
        dbf.setCoalescing(true);

        //  ,       XML-
        dbf.setNamespaceAware(true);

        //         
        DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
        Document doc = documentBuilder.parse(testDoc);

        /*     <ds:Signature>   XML- */

        //           34.10-2012 256
        XMLSignature sig = new XMLSignature(doc, "", signMethod);

        //     
        sig.setId(Id1);

        //    XML-
        Element anElement = doc.getDocumentElement();

        //     XML-   
        anElement.appendChild(sig.getElement());

        /*     XML-         */

        //    <ds:Transforms>  XML-
        Transforms transforms = new Transforms(doc);

        //        
        transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
        transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);
        String[][] filters = {{XPath2FilterContainer.SUBTRACT, "//ds:Signature"}};
        transforms.addTransform(Transforms.TRANSFORM_XPATH2FILTER,
                XPath2FilterContainer.newInstances(doc, filters));

        //       ( <ds:Reference>),    
        // XML- (       <ds:Transforms> 
        //    )
        sig.addDocument("", transforms, digestMethod);

        /*      XML-       ,
            */

        //       <ds:KeyInfo>      
        //    
        sig.addKeyInfo(cert1);

        //    XML-
        sig.sign(privateKey1);

        /*     <ds:Signature>   XML- */

        //           34.10-2012 256
        XMLSignature sigSecond = new XMLSignature(doc, "", signMethod);

        //     
        sigSecond.setId(Id2);

        //     XML-   
        anElement.appendChild(sigSecond.getElement());

        /*     XML-         */

        //    <ds:Transforms>  XML-
        Transforms transforms2 = new Transforms(doc);

        //        
        transforms2.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
        transforms2.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);
        transforms2.addTransform(Transforms.TRANSFORM_XPATH2FILTER,
                XPath2FilterContainer.newInstances(doc, filters));

        //       ( <ds:Reference>),    
        // XML- (       <ds:Transforms> 
        //    )
        sigSecond.addDocument("", transforms2, digestMethod);

        /*      XML-       ,
            */

        //       <ds:KeyInfo>      
        //    
        sigSecond.addKeyInfo(cert2);

        //    XML-
        sigSecond.sign(privateKey2);

        /*   XML-   */

        //  ,      XML-
        FileOutputStream os = new FileOutputStream(signDoc);

        //     XML-  
        TransformerFactory tf = TransformerFactory.newInstance();

        //     XML-  
        Transformer trans = tf.newTransformer();

        //   XML-  
        trans.transform(new DOMSource(doc), new StreamResult(os));
        os.close();
    }

    /**
     *   XML security   
     * @param config   XML security
     * @throws Exception
     */
    static void init(String config) throws Exception {
        System.setProperty(Consts.PROPERTY_NAME, config);
        org.apache.xml.security.Init.init();
    }

    /**
     *     
     * @param ksType  : "CurrentUser" | "LocalComputer" | "CurrentUser/Container" ...
     * @return 
     * @throws Exception
     */
    static KeyStore OpenKeyStore(String ksType) throws Exception {
        KeyStore ks = KeyStore.getInstance("CryptoProCSPKeyStore", "DIGT");
        InputStream store_stream = new ByteArrayInputStream(ksType.getBytes("UTF-8"));
        ks.load(store_stream, null);
        return ks;
    }

    /**
     *     
     * @param ks  
     * @param pair  
     */
    static void DeleteKey(KeyStore ks, KeyPair pair) {
        String alias = ((GOST3410PrivateKey) pair.getPrivate()).getX();
        try {
            ks.deleteEntry(alias);
                    System.out.println("KeyPair from " + alias + " deleted.");
        } catch (Exception ex) {
            System.out.println("KeyPair from " + alias + " is not deleted.");
        }
    }

    /**
     *   
     * @param KeyPairAlgorithm   
     * @param SignAlgorithmName  
     * @param alias    
     * @return
     */
    static KeyPair GeneratePair(
            String KeyPairAlgorithm,
            String SignAlgorithmName,
            String alias) {

        KeyPair pair = null;
        // Generating key pair
        KeyPairGenerator keyGen;
        try {
            keyGen = KeyPairGenerator.getInstance(KeyPairAlgorithm);

            // Key container is specified. Under UNIX it has
            // "\\\\.\\hdimage\\<Container name>" format.
            // Please refer to CSP docs for details
            if (System.getProperty("os.name").toLowerCase().indexOf("windows") < 0) {
                String al = "\\\\.\\hdimage\\" + alias;
                // We should specify some password on UNIX
                keyGen.initialize(new GOST3410KeyGenParameterSpec(al, "11"));
            } else {
                // Under win it's not required to specify password.
                // CSP handles it automatically over GUI.
                keyGen.initialize(new GOST3410KeyGenParameterSpec(alias));
            }
            pair = keyGen.generateKeyPair();

        } catch (Exception ex) {
            //ex.printStackTrace();
        }
        return pair;
    }

    /**
     *     
     * @param pair  
     * @param algorithm  
     * @return
     */
    static X509Certificate generateCert(KeyPair pair, String algorithm) {
        // Generating self-signed certificate
        // Date firstDate, lastDate;
        X509Certificate cert = null;

        Hashtable attrs = new Hashtable();

        // Organization info should be specified here
        attrs.put(X509Principal.C, "RU");
        attrs.put(X509Principal.O, "DIGT");
        attrs.put(X509Principal.L, "Yosh");
        attrs.put(X509Principal.E, "digt@digt.com");

        X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();

        certGen.setSerialNumber(BigInteger.valueOf(567856785));
        certGen.setIssuerDN(new X509Principal(attrs));
        certGen.setNotBefore(new Date(System.currentTimeMillis() - 50000));
        certGen.setNotAfter(new Date(System.currentTimeMillis() + 50000));
        certGen.setSubjectDN(new X509Principal(attrs));
        certGen.setPublicKey(pair.getPublic());
        certGen.setSignatureAlgorithm(algorithm);

        try {
            cert = certGen.generateX509Certificate(pair.getPrivate());
        } catch (Exception ex) {
            //ex.printStackTrace();
        }

        return cert;
    }
}
