/*
* Copyright(C) 1999-2018  << >>
*
*    , 
*    << >>.
*
*        ,
* ,    ,
*     ,
* ,     
*     
*     << >>.
*/

package com.digt.trusted.xmlsig.test;

import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.utils.Constants;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.security.PublicKey;
import java.security.Security;
import java.security.cert.X509Certificate;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.xmlsig.Consts;
import java.io.File;

/**
 *  XML-  
 *   "Xml2Sig_tj_12256.xml"     
 *
 * @author algv
 */
public class XmlSigDocVerify12256 {

    private XmlSigDocVerify12256() {
    }

    /**
     *
     * @param args   XML-,  .
     */
    public static void main(String[] args) {
        try {
            Security.addProvider(new DIGTProvider());

            // xml 
            String signedDoc = "Xml2Sig_tj_12256.xml";

            /*
             *      
             * 
             *
             *    MSXML 5.0,  JCP (   Straight)
            init(Consts.CONFIG);
             *   (   Straight)
            init("resource/tj.xml");
             */
            init(Consts.CONFIG);

            //   
            if (args.length == 0) {
                args = new String[1];
                args[0] = signedDoc;
            }
            for (int i = 0; i < args.length; i++) {
                signedDoc = args[i];
                if ((new File(signedDoc)).exists()) {
                    System.out.print("Verify sign of XML-file: " + signedDoc + " ... ");
                    signDocVer(signedDoc);
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *   XML security   
     * @param config   XML security
     * @throws Exception
     */
    static void init(String config) throws Exception {
        System.setProperty(Consts.PROPERTY_NAME, config);
        org.apache.xml.security.Init.init();
    }

    /**
     *  XML-  .
     *
     * @param signDoc  
     * @return
     * @throws Exception /
     */
    static boolean signDocVer(String signDoc) throws Exception {

        boolean result = false;
        //    XML-
        final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

        //  ,         XML-
        dbf.setIgnoringElementContentWhitespace(true);

        //  ,    CDATA      XML-
        dbf.setCoalescing(true);

        //  ,       XML-
        dbf.setNamespaceAware(true);

        //         
        final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();

        /*   XML-   */

        final Document doc = documentBuilder.parse(signDoc);

        /*    <ds:Signature>  XML- */

        //        Signature
        final Element nscontext = doc.createElementNS(null, "namespaceContext");
        nscontext.setAttributeNS("http://www.w3.org/2000/xmlns/",
                "xmlns:" + "ds".trim(), Constants.SignatureSpecNS);

        //         <ds:Signature>
        final Element sigElement = (Element) XPathAPI.selectSingleNode(doc, "//ds:Signature[1]", nscontext);

        /*   XML-      ,  
        XML- */

        //    
        final XMLSignature signature = new XMLSignature(sigElement, "");

        //   <ds:KeyInfo>    
        final KeyInfo ki = signature.getKeyInfo();

        //        
        final X509Certificate certKey = ki.getX509Certificate();

        //   ,   
        //    
        if (certKey != null) {
            result = signature.checkSignatureValue(certKey);
            System.out.println("The XML signature  is "+(result ? "valid (good)" : "invalid (bad)"));

        } //        
        else {
            //         
            final PublicKey pk = ki.getPublicKey();

            //    ,      
            if (pk != null) {
                result = signature.checkSignatureValue(pk);
                System.out.println("The XML signature  is " + (result ? "valid (good)" : "invalid (bad)"));
            } //        
            else {
                throw new Exception(
                        "There are no information about public key. Verification couldn't be implemented");
            }

        }
        return result;
    }
}
