/*
 * Copyright(C) 1999-2008  << >>
 *
 *    , 
 *    << >>.
 *
 *        ,
 * ,    ,
 *     ,
 * ,     
 *     
 *     << >>.
 */
package com.digt.trusted.xmlsig.test;


import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.algorithms.SignatureAlgorithm;
import org.apache.xml.security.algorithms.JCEMapper;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.signature.ObjectContainer;
import org.apache.xml.security.utils.Constants;
import org.apache.xml.security.utils.XMLUtils;
import org.apache.xpath.XPathAPI;

import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.security.cert.X509Certificate;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Security;
import java.util.Date;
import java.util.Hashtable;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.X509Principal;
import com.digt.trusted.crypto.params.GOST3410KeyGenParameterSpec;
import com.digt.trusted.jce.interfaces.GOST3410PrivateKey;
import com.digt.trusted.x509.X509V3CertificateGenerator;

/**
 *      XML-
 *       (URI)
 *
 * @author vbl
 */
public class XmlSigTest {

    // Enable this for GOST3410-1994
    //private static final String algUri = "http://www.w3.org/2001/04/xmldsig-more#gostr34101994-gostr3411";
    //private static final String sigClass = "com.digt.trusted.xmlsig.SignatureGostR341094GostR3411";
    private static final String digUri = "http://www.w3.org/2001/04/xmldsig-more#gostr3411";
    // Enable this for GOST3410-2001
    private static final String algUri = "http://www.w3.org/2001/04/xmldsig-more#gostr34102001-gostr3411";
    //private static final String sigClass = "com.digt.trusted.xmlsig.SignatureGostR34102001GostR3411";
    // Use this for straight signature byte order witch is compatible with MSXML 5.0
    private static final String sigClass = "com.digt.trusted.xmlsig.SignatureStraightGostR34102001GostR3411";

    private static final String keyalias = "GOST3410EL-XmlSigTest";

    static void init() throws Exception {

        Security.addProvider(new DIGTProvider());

        // Registering an xmlsec algorithm extension
        Class.forName(sigClass);
        SignatureAlgorithm.providerInit();
        SignatureAlgorithm.register(algUri, sigClass);

        // Preparing algorithm mapping for JCEMapper
        String NameSpace = "http://www.xmlsecurity.org/NS/#configuration";
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document doc = dbf.newDocumentBuilder().newDocument();
        Element root = doc.createElementNS(NameSpace, "JCEAlgorithmMappings");
        Element algs = doc.createElementNS(NameSpace, "Algorithms");
        Element el1 = doc.createElementNS(NameSpace, "Algorithm");
        el1.setAttribute("URI", digUri);
        el1.setAttribute("Description", "GOST R 3411 Digest");
        el1.setAttribute("AlgorithmClass", "MessageDigest");
        el1.setAttribute("RequirementLevel", "OPTIONAL");
        el1.setAttribute("JCEName", "GOST3411");
        algs.appendChild(el1);
        Element el2 = doc.createElementNS(NameSpace, "Algorithm");
        el2.setAttribute("URI", algUri);
        el2.setAttribute("AlgorithmClass", "Signature");
        el2.setAttribute("RequirementLevel", "OPTIONAL");
        // Enable this for GOST3410-2001
        el2.setAttribute("JCEName", "GOST3411withGOST3410EL");
        el2.setAttribute("Description", "GOST R 34102001 Digital Signature Algorithm with GOST R 3411 Digest");

        // Enable this for GOST3410-1994
        //el2.setAttribute("JCEName", "GOST3411withGOST3410");
        //el2.setAttribute("Description", "GOST R 341094 Digital Signature Algorithm with GOST R 3411 Digest");

        algs.appendChild(el2);
        root.appendChild(algs);
        doc.appendChild(root);

        // Mandatory initialization of xmlsec
        org.apache.xml.security.Init.init();

        JCEMapper.init(root);

    }

    /**
     *
     * @param doc
     * @param cert
     * @param privateKey
     * @throws Exception
     */
    static void sign(Document doc, X509Certificate cert, PrivateKey privateKey) throws Exception {
        // Constructing an XML document
        // containing XML test text node and signature        
        XMLSignature sig = new XMLSignature(doc, "", algUri);
        doc.appendChild(sig.getElement());

        ObjectContainer obj = new ObjectContainer(doc);
        Element anElement = doc.createElement("TestNode");

        anElement.appendChild(doc.createTextNode("A sample text node"));
        obj.appendChild(anElement);

        String Id = "SignedDocument";

        obj.setId(Id);
        sig.appendObject(obj);

        Transforms transforms = new Transforms(doc);
        transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
        transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);

        sig.addDocument("", transforms, digUri);

        sig.addKeyInfo(cert);

        sig.sign(privateKey);
    }

    /**
     *
     * @param doc
     * @throws Exception
     */
    static void verify(Document doc) throws Exception {
        Element nscontext = XMLUtils.createDSctx(doc, "ds", Constants.SignatureSpecNS);

        Element sigElement = (Element) XPathAPI.selectSingleNode(doc,
                "//ds:Signature[1]", nscontext);
        XMLSignature signature = new XMLSignature(sigElement, "");


        KeyInfo ki = signature.getKeyInfo();

        if (ki != null) {

            if (ki.containsX509Data()) {
                System.out.println("X509Data element found");
            }

            X509Certificate cert = signature.getKeyInfo().getX509Certificate();

            if (cert != null) {

                System.out.println("The XML signature  is "
                        + (signature.checkSignatureValue(cert)
                        ? "valid"
                        : "invalid"));
            } else {
                System.out.println("Did not find a Certificate");

                PublicKey pk = signature.getKeyInfo().getPublicKey();

                if (pk != null) {

                    System.out.println(
                            "Trying to verify the signature using the public key: "
                            + pk);

                    System.out.println("The XML signature is "
                            + (signature.checkSignatureValue(pk)
                            ? "valid"
                            : "invalid"));
                } else {
                    throw new Exception("Public key is not found");
                }
            }
        } else {
            throw new Exception("KeyInfo is not found");
        }
    }

    /**
     *
     * @param args
     */
    public static void main(String[] args) {
        // xml 
        String xmlsigfile = "XmlDSigObject.xml";

        /*
         * Enable this for GOST3410-2001
         *
        String KeyPairAlgorithm = "GOST3410EL";
        String SignAlgorithmName = "GOST3411withGOST3410EL";
         *
         * or enable this for GOST3410-1994
         *
        String KeyPairAlgorithm = "GOST3410";
        String SignAlgorithmName = "GOST3411withGOST3410";
         */
        String KeyPairAlgorithm = "GOST3410EL";
        String SignAlgorithmName = "GOST3411withGOST3410EL";

        try {
            // Framework initialization
            init();

            //   
            System.out.print("Generating keypair ... ");
            KeyPair pair = GeneratePair(KeyPairAlgorithm, SignAlgorithmName, keyalias);
            if (pair != null) {
                System.out.println("Done.");
            } else {
                System.out.println("KeyPair isn`t generated");
                return;
            }

            //  
            System.out.print("Generating certificate ... ");
            X509Certificate cert = generateCert(pair, SignAlgorithmName);
            if (cert != null) {
                System.out.println("Done.");
            } else {
                System.out.println("Certificate isn`t generated");
                return;
            }

            System.out.println("Start signing file ...");
            // Creating a document to be signed
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            Document doc = dbf.newDocumentBuilder().newDocument();

            sign(doc, cert, pair.getPrivate());

            // Creating signed document
            OutputStream os;
            os = new FileOutputStream(xmlsigfile);

            TransformerFactory tf = TransformerFactory.newInstance();
            Transformer trans = tf.newTransformer();
            trans.transform(new DOMSource(doc), new StreamResult(os));
            os.close();
            System.out.println("Created file: " + xmlsigfile);

            // Validating signature
            System.out.print("Start validating signature ... ");
            doc = dbf.newDocumentBuilder().parse(xmlsigfile);
            verify(doc);

            KeyStore ks = OpenKeyStore("CurrentUser/Containers");
            DeleteKey(ks, pair);

        } catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }

    /**
     *     
     * @param ksType  : "CurrentUser" | "LocalComputer" | "CurrentUser/Container" ...
     * @return
     * @throws Exception
     */
    static KeyStore OpenKeyStore(String ksType) throws Exception {
        KeyStore ks = KeyStore.getInstance("CryptoProCSPKeyStore", "DIGT");
        InputStream store_stream = new ByteArrayInputStream(ksType.getBytes("UTF-8"));
        ks.load(store_stream, null);
        return ks;
    }

    /**
     *     
     * @param ks  
     * @param pair  
     */
    static void DeleteKey(KeyStore ks, KeyPair pair) {
        String alias = ((GOST3410PrivateKey) pair.getPrivate()).getX();
        try {
            ks.deleteEntry(alias);
                    System.out.println("KeyPair from " + alias + " deleted.");
        } catch (Exception ex) {
            System.out.println("KeyPair from " + alias + " is not deleted.");
        }
    }

    /**
     *   
     * @param KeyPairAlgorithm   
     * @param SignAlgorithmName  
     * @param alias    
     * @return
     */
    static KeyPair GeneratePair(
            String KeyPairAlgorithm,
            String SignAlgorithmName,
            String alias) {

        KeyPair pair = null;
        // Generating key pair
        KeyPairGenerator keyGen;
        try {
            keyGen = KeyPairGenerator.getInstance(KeyPairAlgorithm);

            // Key container is specified. Under UNIX it has
            // "\\\\.\\hdimage\\<Container name>" format.
            // Please refer to CSP docs for details
            if (System.getProperty("os.name").toLowerCase().indexOf("windows") < 0) {
                String al = "\\\\.\\hdimage\\" + alias;
                // We should specify some password on UNIX
                keyGen.initialize(new GOST3410KeyGenParameterSpec(al, "11"));
            } else {
                // Under win it's not required to specify password.
                // CSP handles it automatically over GUI.
                keyGen.initialize(new GOST3410KeyGenParameterSpec(alias));
            }
            pair = keyGen.generateKeyPair();

        } catch (Exception ex) {
            //ex.printStackTrace();
        }
        return pair;
    }

    /**
     *     
     * @param pair  
     * @param algorithm  
     * @return
     */
    static X509Certificate generateCert(KeyPair pair, String algorithm) {
        // Generating self-signed certificate
        // Date firstDate, lastDate;
        X509Certificate cert = null;

        Hashtable attrs = new Hashtable();

        // Organization info should be specified here
        attrs.put(X509Principal.C, "RU");
        attrs.put(X509Principal.O, "DIGT");
        attrs.put(X509Principal.L, "Yosh");
        attrs.put(X509Principal.E, "digt@digt.com");

        X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();

        certGen.setSerialNumber(BigInteger.valueOf(567856785));
        certGen.setIssuerDN(new X509Principal(attrs));
        certGen.setNotBefore(new Date(System.currentTimeMillis() - 50000));
        certGen.setNotAfter(new Date(System.currentTimeMillis() + 50000));
        certGen.setSubjectDN(new X509Principal(attrs));
        certGen.setPublicKey(pair.getPublic());
        certGen.setSignatureAlgorithm(algorithm);

        try {
            cert = certGen.generateX509Certificate(pair.getPrivate());
        } catch (Exception ex) {
            //ex.printStackTrace();
        }

        return cert;
    }
}
