/*
 * Copyright(C) 1999-2008  << >>
 *
 *    , 
 *    << >>.
 *
 *        ,
 * ,    ,
 *     ,
 * ,     
 *     
 *     << >>.
 */
package com.digt.trusted.xmlsig.test;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.security.cert.X509Certificate;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.Security;
import java.util.Date;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;

import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.signature.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import com.digt.trusted.jce.provider.DIGTProvider;
import com.digt.trusted.jce.X509Principal;
import com.digt.trusted.crypto.params.GOST3410KeyGenParameterSpec;
import com.digt.trusted.jce.interfaces.GOST3410PrivateKey;
import com.digt.trusted.x509.X509V3CertificateGenerator;
import com.digt.trusted.xmlsig.Consts;

/**
 *    XML-
 *       (URN)
 *
 * @author vbl
 */
public class XmlSigTestURN {
    /*
     * Digest GOST3411
     * uri
    private static final String digUri = Consts.URI_GOST_DIGEST;
     * or urn:
    private static final String digUri = Consts.URN_GOST_DIGEST;
     */

    private static final String digUri = Consts.URN_GOST_DIGEST;

    /*
     * Signature GOST3410-1994
     * uri
    private static final String algUri = Consts.URI_GOST_SIGN_94;

     *
     * Signature GOST3410-2001
     * uri
    private static final String algUri = Consts.URI_GOST_SIGN;
     * or urn:
    private static final String algUri = Consts.URN_GOST_SIGN;
     */
    private static final String algUri = Consts.URN_GOST_SIGN;
    private static final String keyalias = "GOST3410EL-XMLSigTestURN";

    /**
     * 
     * @param args
     */
    public static void main(String[] args) {
        // xml 
        String xmlfile = "data/Xml2Sig.xml";

        // xml 
        String xmlsigfile = "Xml2Sig_tj.xml";

        /*
         * Enable this for GOST3410-2001
         *
        String KeyPairAlgorithm = "GOST3410EL";
        String SignAlgorithmName = "GOST3411withGOST3410EL";
         *
         * or enable this for GOST3410-1994
         *
        String KeyPairAlgorithm = "GOST3410";
        String SignAlgorithmName = "GOST3411withGOST3410";
         */
        String KeyPairAlgorithm = "GOST3410EL";
        String SignAlgorithmName = "GOST3411withGOST3410EL";

        try {
            Security.addProvider(new DIGTProvider());

            //   
            System.out.print("Generating keypair ... ");
            KeyPair pair = GeneratePair(KeyPairAlgorithm, SignAlgorithmName, keyalias);
            if (pair != null) {
                System.out.println("Done.");
            } else {
                System.out.println("KeyPair isn`t generated");
                return;
            }

            //  
            System.out.print("Generating certificate ... ");
            X509Certificate cert = generateCert(pair, SignAlgorithmName);
            if (cert != null) {
                System.out.println("Done.");
            } else {
                System.out.println("Certificate isn`t generated");
                return;
            }

            //        
            if (args.length > 0) {
                String al = args[0];
                if ((new File(al)).exists()) {
                    xmlfile = al;
                }
            }

            /*
             *      
             * 
             *
             *    MSXML 5.0,  JCP (   Straight)
            init(Consts.CONFIG);
             *   (   Straight)
            init("resource/tj.xml");
             */
            init(Consts.CONFIG);

            System.out.println("Start signing file ...");
            signDoc(pair, cert, algUri, digUri, xmlfile, xmlsigfile);
            System.out.println("Created file: " + xmlsigfile);

            KeyStore ks = OpenKeyStore("CurrentUser/Containers");
            DeleteKey(ks, pair);

        } catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }

    /**
     *    XML-   
     *
     * @param keypair
     * @param cert  
     * 
     * @param signMethod  
     * @param digestMethod  ,   
     * @param testDoc  
     * @param signDoc  
     * @throws Exception e
     */
    static void signDoc(KeyPair keypair, X509Certificate cert,
            String signMethod, String digestMethod,
            String testDoc, String signDoc)
            throws Exception {

        /*   XML-   */

        //    XML-
        final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

        //  ,         XML-
        dbf.setIgnoringElementContentWhitespace(true);

        //  ,    CDATA      XML-
        dbf.setCoalescing(true);

        //  ,       XML-
        dbf.setNamespaceAware(true);

        //         
        final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
        final Document doc = documentBuilder.parse(testDoc);

        /*    <ds:Signature>   XML- */

        //           34.10-2001
        final XMLSignature sig = new XMLSignature(doc, "", signMethod);

        //    XML-
        final Element anElement = doc.getDocumentElement();

        //     XML-  
        anElement.appendChild(sig.getElement());

        /*     XML-        */

        //    <ds:Transforms>  XML-
        final Transforms transforms = new Transforms(doc);

        //        
        transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
        transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);

        //      ( <ds:Reference>),    
        // XML- (       <ds:Transforms> 
        //    )
        sig.addDocument("", transforms, digestMethod);

        /*     XML-
         *    ,    
         *
         *      <ds:KeyInfo>
         *       
         *
         */
        sig.addKeyInfo(cert);

        //   
        final PrivateKey privateKey = keypair.getPrivate();
        //   XML-
        sig.sign(privateKey);

        /*   XML-   */

        //  ,      XML-
        final FileOutputStream os = new FileOutputStream(signDoc);

        //     XML-  
        final TransformerFactory tf = TransformerFactory.newInstance();

        //     XML-  
        final Transformer trans = tf.newTransformer();

        //   XML-  
        trans.transform(new DOMSource(doc), new StreamResult(os));
        os.close();
    }

    /**
     *   XML security   
     * @param config   XML security
     * @throws Exception
     */
    static void init(String config) throws Exception {
        System.setProperty(Consts.PROPERTY_NAME, config);
        org.apache.xml.security.Init.init();
    }

    /**
     *     
     * @param ksType  : "CurrentUser" | "LocalComputer" | "CurrentUser/Container" ...
     * @return
     * @throws Exception
     */
    static KeyStore OpenKeyStore(String ksType) throws Exception {
        KeyStore ks = KeyStore.getInstance("CryptoProCSPKeyStore", "DIGT");
        InputStream store_stream = new ByteArrayInputStream(ksType.getBytes("UTF-8"));
        ks.load(store_stream, null);
        return ks;
    }

    /**
     *     
     * @param ks  
     * @param pair  
     */
    static void DeleteKey(KeyStore ks, KeyPair pair) {
        String alias = ((GOST3410PrivateKey) pair.getPrivate()).getX();
        try {
            ks.deleteEntry(alias);
            System.out.println("KeyPair from " + alias + " deleted.");
        } catch (Exception ex) {
            System.out.println("KeyPair from " + alias + " is not deleted.");
        }
    }

    /**
     *   
     * @param KeyPairAlgorithm   
     * @param SignAlgorithmName  
     * @param alias    
     * @return
     */
    static KeyPair GeneratePair(
            String KeyPairAlgorithm,
            String SignAlgorithmName,
            String alias) {

        KeyPair pair = null;
        // Generating key pair
        KeyPairGenerator keyGen;
        try {
            keyGen = KeyPairGenerator.getInstance(KeyPairAlgorithm);

            // Key container is specified. Under UNIX it has
            // "\\\\.\\hdimage\\<Container name>" format.
            // Please refer to CSP docs for details
            if (System.getProperty("os.name").toLowerCase().indexOf("windows") < 0) {
                String al = "\\\\.\\hdimage\\" + alias;
                // We should specify some password on UNIX
                keyGen.initialize(new GOST3410KeyGenParameterSpec(al, "11"));
            } else {
                // Under win it's not required to specify password.
                // CSP handles it automatically over GUI.
                keyGen.initialize(new GOST3410KeyGenParameterSpec(alias));
            }
            pair = keyGen.generateKeyPair();

        } catch (Exception ex) {
            //ex.printStackTrace();
        }
        return pair;
    }

    /**
     *     
     * @param pair  
     * @param algorithm  
     * @return
     */
    static X509Certificate generateCert(KeyPair pair, String algorithm) {
        // Generating self-signed certificate
        // Date firstDate, lastDate;
        X509Certificate cert = null;

        Hashtable attrs = new Hashtable();

        // Organization info should be specified here
        attrs.put(X509Principal.C, "RU");
        attrs.put(X509Principal.O, "DIGT");
        attrs.put(X509Principal.L, "Yosh");
        attrs.put(X509Principal.E, "digt@digt.com");

        X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();

        certGen.setSerialNumber(BigInteger.valueOf(567856785));
        certGen.setIssuerDN(new X509Principal(attrs));
        certGen.setNotBefore(new Date(System.currentTimeMillis() - 50000));
        certGen.setNotAfter(new Date(System.currentTimeMillis() + 50000));
        certGen.setSubjectDN(new X509Principal(attrs));
        certGen.setPublicKey(pair.getPublic());
        certGen.setSignatureAlgorithm(algorithm);

        try {
            cert = certGen.generateX509Certificate(pair.getPrivate());
        } catch (Exception ex) {
            //ex.printStackTrace();
        }

        return cert;
    }
}
